//    WeIRCd administration tool
//    Copyright (C) 2009-2010 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "admintool.h"

void gsetvalue(struct settingstab* s, const char* name, const char* value) {
	if (value == NULL) {return;}
	//printf("trying to set '%s' to '%s'\n",name,value);
	void* item = s->settingssubtabslist;
	while (item != NULL) {
		struct settingssubtab* t = linkedlist_GetData(item);
		void* item2 = t->valueslist;
		while (item2 != NULL) {
			struct settingsvalue* v = linkedlist_GetData(item2);
			if (v->name != NULL) {
				if (strcasecmp(v->name,name) == 0) {
					if (v->type == VARTYPE_ENTRY) {
						gtk_entry_set_text(GTK_ENTRY(v->settingswidget),value);
					}
					if (v->type == VARTYPE_INTCOUNT) {
						double min;double max;
						gtk_spin_button_get_range(GTK_SPIN_BUTTON(v->settingswidget),
						&min,&max);
						int intval = atoi(value);
						if (intval <= 0) {return;}
						if (intval < min) {intval = min;}
						if (intval > max) {intval = max;}
						gtk_spin_button_set_value(GTK_SPIN_BUTTON(v->settingswidget),
						intval);
					}
					if (v->type == VARTYPE_COMBOBOX) {
						GtkTreeModel* m = gtk_combo_box_get_model(GTK_COMBO_BOX(v->settingswidget));
						GtkTreePath* t = gtk_tree_path_new_first();
						GtkTreeIter i;
						if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(v->settingswidget), &i) == FALSE) {
							if (gtk_tree_model_get_iter(m, &i, t) == FALSE) {return;}
							gtk_combo_box_set_active_iter(GTK_COMBO_BOX(v->settingswidget),&i);
						}
					}
				}
			}
			item2 = linkedlist_NextItem(item2);
		}
		item = linkedlist_NextItem(item);
	}
}
char getvalbuf[200];
const char* ggetvalue(struct settingstab* s, const char* name) {
	printf("trying to get '%s'\n",name);
	void* item = s->settingssubtabslist;
	while (item != NULL) {
		struct settingssubtab* t = linkedlist_GetData(item);
		void* item2 = t->valueslist;
		while (item2 != NULL) {
			struct settingsvalue* v = linkedlist_GetData(item2);
			if (v->name != NULL) {
				if (strcasecmp(v->name,name) == 0) {
					if (v->type == VARTYPE_ENTRY) {
						return gtk_entry_get_text(GTK_ENTRY(v->settingswidget));
					}
					if (v->type == VARTYPE_INTCOUNT) {
						snprintf(getvalbuf,199,"%d",(int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(v->settingswidget)));
						return getvalbuf;
					}
					if (v->type == VARTYPE_COMBOBOX) {
						GtkTreeModel* m = gtk_combo_box_get_model(GTK_COMBO_BOX(v->settingswidget));
						GtkTreePath* t = gtk_tree_path_new_first();
						GtkTreeIter i;
						
						if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(v->settingswidget), &i) == FALSE) {strcpy(getvalbuf,"");return getvalbuf;}
						GValue val;memset(&val,0,sizeof(val));
						
						gtk_tree_model_get_value(m, &i, 0, &val);
						strcopy(getvalbuf,g_value_get_string(&val),199);
						g_value_unset(&val);
						if (strendswith(getvalbuf,"(recommended)") == 0) {//strip of (recommended) if it's appended
							getvalbuf[strlen(getvalbuf)-strlen("(recommended)")] = 0;
							trimright(getvalbuf);
						}
						return getvalbuf;
					}
					return;
				}
			}
			item2 = linkedlist_NextItem(item2);
		}
		item = linkedlist_NextItem(item);
	}
	strcpy(getvalbuf,"");
	return getvalbuf;
}

void transfer_configtogui_fortab(struct settingstab* s) {
	s->dontwatchchanges = 1; //make GUI not respond to changes by enabling apply/discard buttons
	printf("now. transfer...\n");
	printf("btw. server#name is: %s\n",value_get("server#name"));
	//general
	gtk_widget_set_sensitive(s->applybutton,FALSE);
	gtk_widget_set_sensitive(s->discardbutton, FALSE);
	gsetvalue(s,"name",value_get("server#name"));
	gsetvalue(s,"description",value_get("server#description"));
	gsetvalue(s,"network",value_get("server#network"));
	gsetvalue(s,"password",value_get("server#password"));
	gsetvalue(s,"maxchansperuser",value_get("server#maxchansperuser"));
	gsetvalue(s,"helpchan",value_get("server#helpchan"));
	gsetvalue(s,"motdfile",value_get("server#motdfile"));
	gsetvalue(s,"glinefile",value_get("server#glinefile"));
	gsetvalue(s,"klinefile",value_get("server#klinefile"));
	
	//flood protection
	gsetvalue(s,"level",value_get("floodprotect#level"));
	gsetvalue(s,"MaxConnsPerIp",value_get("floodprotect#maxconnsperip"));
	gsetvalue(s,"resolvedhosts",value_get("floodprotect#ResolvedHosts"));
	gsetvalue(s,"connectthrottling",value_get("floodprotect#ConnectThrottling"));
		
	s->dontwatchchanges = 0;
	
}
void transfer_guitoconfig(struct settingstab* s) {
	//general
	value_set("server#name",ggetvalue(s,"name"));
	value_set("server#description",ggetvalue(s,"description"));
	value_set("server#network",ggetvalue(s,"network"));
	value_set("server#password",ggetvalue(s,"password"));
	value_set("server#maxchansperuser",ggetvalue(s,"maxchansperuser"));
	value_set("server#helpchan",ggetvalue(s,"helpchan"));
	value_set("server#motdfile",ggetvalue(s,"motdfile"));
	value_set("server#glinefile",ggetvalue(s,"glinefile"));
	value_set("server#klinefile",ggetvalue(s,"klinefile"));
	
	//flood protection
	value_set("floodprotect#level",ggetvalue(s,"level"));
	value_set("floodprotect#maxconnsperip",ggetvalue(s,"MaxConnsPerIp"));
	value_set("floodprotect#ResolvedHosts",ggetvalue(s,"resolvedhosts"));
	value_set("floodprotect#ConnectThrottling",ggetvalue(s,"connectthrottling"));
}
void transfer_configtogui(struct settingstab* stab) {
	if (stab != NULL) {
		transfer_configtogui_fortab(stab);return;
	}
	void* item = tablist;
	while (item != NULL) {
		struct tab* t = linkedlist_GetData(item);
		
		struct settingstab* s = &t->ct.s;
		
		transfer_configtogui_fortab(s);
		item = linkedlist_NextItem(item);
	}
	
}
