#!/bin/sh
bindir="/usr/bin"
datadir="/usr/local/var/weircd"
configdir="/etc/weircd/"

cs="\E[47;31m"
ce="\033[0m"

echo "#-._.-'°'-._.-'°'-._.-'°'-._.-'°'-._.-'°'-._.-#"
if ! type -P autoconf &>/dev/null; then
echo "Cannot continue installation:"
echo "Please install autoconf."
exit
fi
if ! type -P automake &>/dev/null; then
echo "Cannot continue installation:"
echo "Please install automake."
exit
fi
if ! type -P libtool &>/dev/null; then
echo "Cannot continue installation:"
echo "Please install libtool."
exit
fi
echo "  Welcome to the WeIRCd 0.8.2 Unix setup wizard."
echo "  This shell script will ask you for the most"
echo "  important system paths and settings."
echo " "
echo " IF YOU ARE UNSURE: You may always just press enter and the default"
echo "  will be used."
echo " "
echo "  Bin dir (the place where the executable resides)"
echo "    [ENTER PATH or press enter to use $bindir]:"
read -r newbindir
count=`echo "$newbindir" | wc -c`
if [ $count -gt 1 ]; then
bindir=$newbindir
fi
echo " "
echo "  Data dir (the place where runtime data is saved to)"
echo "    [ENTER PATH or press enter to use $datadir]:"
read -r newdatadir
count=`echo "$newdatadir" | wc -c`
if [ $count -gt 1 ]; then
datadir=$newdatadir
fi
echo " "
echo "  Config dir (the place where the configuration file resides)"
echo "    [ENTER PATH or press enter to use $configdir]:"
read -r newconfigdir
count=`echo "$newconfigdir" | wc -c`
if [ $count -gt 1 ]; then
configdir=$newconfigdir
fi
echo " "
echo "  If you want to cross compile, enter a specific"
echo "  host target now (otherwise just press enter):"
read -r specialhosttarget

echo " Thanks!"
echo " "
echo "SUMMARY"
echo "     Bin dir:     $bindir"
echo "     Data dir:    $datadir"
echo "     Config dir:  $configdir"
count=`echo "$specialhosttarget" | wc -c`
if [ $count -gt 1 ]; then
echo -e "     Host target: $specialhosttarget"
hostoption="--host=$specialhosttarget"
else
echo "     Host target: [Default]"
hostoption=""
fi
echo "  Is this OK? [Enter Y to continue, nothing to cancel]:"
read -r answer
case "$answer" in
[yY] )
echo " Nice!"
;;
*   )
exit
;;
esac

echo "     "
echo " "
echo "INSTALL: Now generating configure files ..."

glibtoolize &> /dev/null
libtoolize &> /dev/null
aclocal
autoconf
automake --add-missing
glibtoolize &> /dev/null
libtoolize &> /dev/null

echo " Done."
echo " "
echo "INSTALL: Executing ./configure with proper values ..."
sh ./configure --bindir=${bindir} --sysconfdir=${configdir} --localstatedir=${datadir} --libdir=${datadir}/modules $hostoption
retval=$?
if [ $retval -gt 0 ]; then
echo "THERE WENT SOMETHING WRONG."
echo "  Apparently, your system is missing something vital for WeIRCd"
echo "  to compile."
echo "  Please check that you have sqlite3 installed (which is needed"
echo "  for the built-in services), that the dl lib is available and"
echo "  that you have a C compiler installed!"
echo "  "
echo "  If you are unable to fix those problems and you're on a system"
echo "  that is normally supported by WeIRCd (any Unix that is not too"
echo "  exotic and supported by GNU build tools), get help here:"
echo "       http://eloxoph.com/weircd/support"
exit
fi

echo "SUCCESS."
echo " "
echo " Do you now wish to compile WeIRCd now?"
echo " You may also do this later using \"make\"."
echo "  [Enter N to cancel]"
read -r answer
case "$answer" in
[nN] )
exit
;;
*   )

;;
esac
echo " "
echo "COMPILING..."
echo "  This may take some time."
echo " "
sleep 2

make

retval=$?

if [ $retval -gt 0 ]; then
echo "THERE WENT SOMETHING WRONG."
echo " This shouldn't happen. Consult the following web link for help:"
echo "       http://eloxoph.com/weircd/support"
exit
fi
echo " "
echo "DONE."
echo "  Now the only thing you still need to do to be able to use"
echo "  WeIRCd is to INSTALL it."
echo "  To do that, enter \"sudo make install\"."
