//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

const char* mainblocknames[] = {"Server","Services","NoOpChans","FloodProtect","IrcOps","Link","WebIRC","Listen",NULL};
int mainblocknamevalid(const char* block) {
	int r = 0;
	while (mainblocknames[r] != NULL) {
		if (strcasecmp(mainblocknames[r],block) == 0) {return 0;}
		r++;
	}
	return -1;
}
int subblockvalid(const char* mainblock, const char* subblock) {
	if (strcasecmp(mainblock,"floodprotect") == 0) {
		if (strcasecmp(subblock,"hostcloaking") == 0) {
			return 0;
		}
		if (strcasecmp(subblock,"reservednicks") == 0) {
			return 0;
		}
	}
	if (strcasecmp(mainblock,"ircops") == 0 || strcasecmp(mainblock,"services") == 0) {
		return 0;//allow any sub blocks
	}
	return -1;
}
int cblockparser_Do() {
	char buf[300] = "";
	char msg[300];
	const int maxbufflen = 300;
	int c = creader_GetC();
	while (c != EOF) {
		//level 1 - SEARCH BLOCK
		if (c != '{' && c != ';' && c != '\n') {
			addtobuf(buf,c,maxbufflen);
		}else{
			trimright(buf);trimleft(buf);
			if (c == ';' || c == '\n') {
				if (strlen(buf) <= 0) {
					if (c == '\n') {
						c = creader_GetC();
						continue;
					}
					cerr("Keyword expected",creader_GetLine());
				}else{
					if (strstw(buf,"version=") == 0) {
						strcpy(buf,"");
						c = creader_GetC();
						continue;
					}
					cerr("'{' expected",creader_GetLine());
				}
				return -1;
			}
			if (mainblocknamevalid(buf) != 0) {
				trimright(buf);cutonfirstspace(buf);
				snprintf(msg,299,"'%s' is not a valid block here",buf);
				msg[299] = 0;
				cerr(msg,creader_GetLine());
				return -1;
			}else{
				if (strcasecmp(buf,"webirc") == 0) {
					if (webirc_Add() < 0) {
						cerr(error_mem,creader_GetLine());
						return -1;
					}
				}
				if (strcasecmp(buf,"link") == 0) {
					linkinfo_NewBlock();
				}
			}
			//printf("We are now inside block '%s'\n",buf);
			char mainblock[300];
			strcpy(mainblock,buf);
			strcpy(buf,"");
			c = creader_GetC();
			while (c != EOF) {
				//level 2 - PARSE BLOCK CONTENTS
				if (c != '\n' && c != ';' && c != '}' && c != '{') {
					addtobuf(buf,c,maxbufflen);
				}else{
					trimright(buf);trimleft(buf);
					if (c == '\n') {
						if (subblockvalid(mainblock,buf) == 0) {
							//we are at a block name. do not treat it as keyword and continue parsing until {
							c = creader_GetC();continue;
						}
					}
					if ((c == ';' || c == '{') && strlen(buf) <= 0) {
						cerr("Keyword expected",creader_GetLine());
						return -1;
					}
					if (c == '{') {
						if (subblockvalid(mainblock,buf) != 0) {
							snprintf(msg,299,"'%s' is not a valid block here",buf);
							msg[299] = 0;
							cerr(msg,creader_GetLine());
							return -1;
						}
						//printf("We are now inside sub block '%s'\n",buf);
						if (strcasecmp(mainblock,"ircops") == 0) {
							int r = ircopinfo_Add(buf);
							if (r < 0) {
								if (r == -2) {
									cerr("IRC Operator account double defined",creader_GetLine());
								}
								if (r == -1) {
									cerr(error_mem,creader_GetLine());
								}
								return -1;
							}
						}
						if (strcasecmp(mainblock,"services") == 0) {
							struct moduleinfo* inf;
							/* Load the services module. We will leave it LOADED so while parsing the block we can
							   send options we parsed to it and have them checked. Scroll down for a note about
							   unloading it again */
							inf = module_QueryInfo_NoUnload(buf,&examinemodule,&unloadexaminemodule);
							if (inf == NULL) {
								snprintf(msg,299,"Module '%s' unsuitable/broken: %s",buf,modbreakage[module_ModLoadError()]);
								msg[299] = 0;
								cerr(msg,creader_GetLine());
								
								return -1;
							}
							if (inf->services != 1) {
								snprintf(msg,299,"Module '%s' unsuitable/broken: No services functionality",buf);
								msg[299] = 0;
								cerr(msg,creader_GetLine());
								module_FreeInfo(inf);
								return -1;
							}
							module_FreeInfo(inf);
							servicesmodule_enabledline = 0;
							/* IMPORTANT: This module *will* either be unloaded when the block ends regularly, or by the
							   readconf.c control code that checks after closing the config due to an error if there is
							   still this examined module loaded. */
						}
						char subblock[300];
						strcpy(subblock,buf);
						strcpy(buf,"");
						c = creader_GetC();
						while (c != EOF) {
							//level 3 - PARSE SUB BLOCK CONTENTS
							if (c != '\n' && c != ';' && c != '}' && c != '{') {
								addtobuf(buf,c,maxbufflen);
							}else{
								trimright(buf);trimleft(buf);
								if ((c == ';' || c == '{') && strlen(buf) <= 0) {
									cerr("Keyword expected",creader_GetLine());
									return -1;
								}
								if (c == '{') {//no further sub blocks valid at the moment
									snprintf(msg,299,"'%s' is not a valid block here",buf);
									msg[299] = 0;
									cerr(msg,creader_GetLine());
									return -1;
								}
								if (strlen(buf) > 0) {
									//parse the keyword
									if (clineparser_Do(mainblock,subblock,buf) < 0) {
										return -1;
									}
								}
								strcpy(buf,"");
								if (c == '}') {
									//the sub block is finished -> quit loop
									if (strcasecmp(mainblock,"services") == 0) {
										if (servicesmodule_enabledline == 0) {
											cerr("Services block misses enabled or disabled line",creader_GetLine());
											return -1;
										}
										if (servicesmodule_enabledline == 1) {
											int rval = servicesmodule_FinishConfigPreCheck(modman_GetNameByPtr(examinemodule));
											if (rval != 0) {
												snprintf(msg,299,"Some options missing for '%s'",modman_GetNameByPtr(examinemodule));
												msg[299] = 0;
												cerr(msg,creader_GetLine());
												return -1;
											}
										}
										/* unload a module we probably examined */
										if (examinemodule != NULL && unloadexaminemodule == 1) {
											modman_UnloadByPtr(examinemodule);
										}
										examinemodule = NULL;unloadexaminemodule = 0;
									}
									
									break;
								}
							}
							c = creader_GetC();
						}
						if (c == EOF) {
							cerr("Unexpected end of file",creader_GetLine());
							return -1;
						}
						if (c == '}') {
							c = 0;
						}
					}
					if (strlen(buf) > 0) {
						//parse the keyword
						if (clineparser_Do(mainblock,NULL,buf) < 0) {
							return -1;
						}
					}
					strcpy(buf,"");
					if (c == '}') {
						//the block is finished -> quit loop
						if (strcasecmp(mainblock,"link") == 0) {
							if (linkinfo_EndBlock() != 0) {
								cerr("Link block misses vital option",creader_GetLine());
								return -1;
							}
						}
						break;
					}
				}
				c = creader_GetC();
			}
			//printf("leaving block %s\n",mainblock);
			if (strcasecmp(mainblock,"webirc") == 0) {
				if (webirc_EntryComplete() != 0) {
					cerr("WebIRC block is missing some options",creader_GetLine());
					return -1;
				}
			}
			if (c == EOF) {
				cerr("Unexpected end of file",creader_GetLine());
				return -1;
			}
		}
		c = creader_GetC();
	}
	return 0;
}