//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// this is for saving more complicated config options which are arrays, not single values

#include "weircd.h"

//linked list of NoOpChans
void* noopchanlist = NULL;//first item
void* noopchanlist_new = NULL;

//linked list of ReservedNicks
void* reservednicklist = NULL;//first item
void* reservednicklist_new = NULL;

//linked list of IRC op data
void* ircoplist = NULL;
void* ircoplist_new = NULL;

//linked list of bind ports
void* conflisteners = NULL;
void* conflisteners_new = NULL;

//linked list of webirc
void* webirclist = NULL;
void* webirclist_new = NULL;
void* lastwebirc = NULL;

int webirc_Add() {
	struct webirc w;memset(&w,0,sizeof(w));
	void* s = linkedlist_Append(&webirclist_new,&w,sizeof(w));
	if (s == NULL) {
		return -1;
	}else{
		lastwebirc = s;
		return 0;
	}
}
int webirc_SetType(const char* type) {
	if (lastwebirc == NULL) {return -1;}
	struct webirc* w = linkedlist_GetData(lastwebirc);
	if (strcasecmp(w->type,"") != 0) {
		return -1;
	}
	strncpy(w->type,type,MAXWIRCTYPELEN);w->type[MAXWIRCTYPELEN] = 0;
	return 0;
}
int webirc_SetHost(const char* host) {
	if (lastwebirc == NULL) {return -1;}
	struct webirc* w = linkedlist_GetData(lastwebirc);
	if (strcasecmp(w->host,"") != 0) {
		return -1;
	}
	strncpy(w->host,host,IPMAXLEN);w->host[IPMAXLEN] = 0;
	return 0;
}
int webirc_SetIdent(const char* ident) {
	if (lastwebirc == NULL) {return -1;}
	struct webirc* w = linkedlist_GetData(lastwebirc);
	if (strcasecmp(w->ident,"") != 0) {
		return -1;
	}
	strncpy(w->ident,ident,COMPILEOPTION_MAXUSERIDLEN);w->ident[COMPILEOPTION_MAXUSERIDLEN] = 0;
	return 0;
}
int webirc_SetPassword(const char* pw) {
	if (lastwebirc == NULL) {return -1;}
	struct webirc* w = linkedlist_GetData(lastwebirc);
	if (strcasecmp(w->password,"") != 0) {
		return -1;
	}
	strncpy(w->password,pw,WEBIRC_MAXPWLEN);w->password[WEBIRC_MAXPWLEN] = 0;
	return 0;
}
int webirc_EntryComplete() {
	if (lastwebirc == NULL) {return 0;}
	struct webirc* w = linkedlist_GetData(lastwebirc);
	if (strcasecmp(w->password,"") == 0) {return -1;}
	if (strcasecmp(w->host,"") == 0) {return -1;}
	return 0;
}
char staticidentbuffer[COMPILEOPTION_MAXUSERIDLEN+1];
int webirc_AccessAllowed(const char* host, const char* passw, const char* type) {
	void* item = webirclist;
	while (item != NULL) {
		struct webirc* w = linkedlist_GetData(item);
		if (w != NULL) {
			const char* typeS = w->type;
			const char defaulttype[] = "cgiirc";
			if (strlen(typeS) <= 0) {typeS = defaulttype;}
			if (strcasecmp(w->password,passw) == 0 && strcasecmp(w->host,host) == 0 && strcasecmp(typeS,type) == 0) {
				if (w->ident != NULL) {
					strcpy(staticidentbuffer,w->ident);
				}else{
					strcpy(staticidentbuffer,"");
				}
				return 0;
			}
			/*if (strcasecmp(w->password,passw) != 0 && strcasecmp(w->host,host) == 0) {
				return -1;
			}*/
		}
		item = linkedlist_NextItem(item);
	}
	return -1;
}
const char* webirc_GetIdent() {
	if (strlen(staticidentbuffer) <= 0) {return NULL;}
	return staticidentbuffer;
}
int noopchan_Add(const char* chan) {
	if (stringlist_CheckItem(noopchanlist_new,chan) >= 0) {
		return 0;
	}
	void* s = linkedlist_Append(&noopchanlist_new,chan,strlen(chan)+1);
	if (s != NULL) {
		return 0;
	}else{
		return -1;
	}
}
int reservednicklist_Add(const char* chan) {
	if (stringlist_CheckItem(reservednicklist_new,chan) >= 0) {
		return 0;
	}
	void* s = linkedlist_Append(&reservednicklist_new,chan,strlen(chan)+1);
	if (s != NULL) {
		return 0;
	}else{
		return -1;
	}
}
void reservednicklist_ClearNew() {
	linkedlist_Free(&reservednicklist_new);
	reservednicklist_new = NULL;
}
void noopchans_ClearNew() {
	linkedlist_Free(&noopchanlist_new);
	noopchanlist_new = NULL;
}
int ircoplist_CheckItem(void* list, const char* name) {
	void* s = list;
	struct ircopinfo* i;
	while (s != NULL) {
		i = (struct ircopinfo*)linkedlist_GetData(s);
		if (i != NULL) {
			if (strcasecmp(i->name,name) == 0) {
				return 0;
			}
		}
		s = linkedlist_NextItem(s);
	}
	return -1;
}
struct ircopinfo* ircoplist_GetItem(const char* name) {
	void* s = ircoplist;
	struct ircopinfo* i;
	while (s != NULL) {
		i = (struct ircopinfo*)linkedlist_GetData(s);
		if (i != NULL) {
			if (strcmp(i->name,name) == 0) {
				return i;
			}
		}
		s = linkedlist_NextItem(s);
	}
	return NULL;
}
void ircopinfo_FreeInfo(void* listitem) {
	struct ircopinfo* i = (struct ircopinfo*)listitem;
	if (i->name != NULL) {free(i->name);i->name = NULL;}
	if (i->pass != NULL) {free(i->pass);i->pass = NULL;}
	if (i->host != NULL) {free(i->host);i->host = NULL;}
}
void ircopinfo_FreeList(void* list) {
	if (list == NULL) {return;}
	void* s = list;
	while (s != NULL) {
		ircopinfo_FreeInfo(linkedlist_GetData(s));
		s = linkedlist_NextItem(s);
	}
	linkedlist_Free(&list);
}
int ircopinfo_SetHost(const char* opname, const char* host) {
	void* s = ircoplist_new;
	struct ircopinfo* i;
	while (s != NULL) {
		i = (struct ircopinfo*)linkedlist_GetData(s);
		if (i != NULL) {
			if (strcasecmp(i->name,opname) == 0) {
				if (i->host != NULL) {return -2;}
				i->host = malloc(strlen(host)+1);
				if (i->host == NULL) {
					return -1;
				}
				strcpy(i->host,host);
				return 0;
			}
		}
		s = linkedlist_NextItem(s);
	}
	return -1;
}
int ircopinfo_SetFlag(const char* opname, const char* flag) {
	void* s = ircoplist_new;
	struct ircopinfo* i;
	while (s != NULL) {
		i = (struct ircopinfo*)linkedlist_GetData(s);
		if (i != NULL) {
			if (strcasecmp(i->name,opname) == 0) {
				//printf("parsing flag %s for %s\n",flag,i->name);
				if (strcasecmp(flag,"Global") == 0) {
					i->flags.flag_global = 1;
					return 0;
				}
				if (strcasecmp(flag,"Linking") == 0) {
					i->flags.flag_linking = 1;
					return 0;
				}
				if (strcasecmp(flag,"Connections") == 0) {
					i->flags.flag_connectionnotices = 1;
					return 0;
				}
				if (strcasecmp(flag,"Services") == 0) {
					i->flags.flag_services = 1;
					return 0;
				}
				if (strcasecmp(flag,"SetHost") == 0) {
					i->flags.flag_sethost = 1;
					return 0;
				}
				if (strcasecmp(flag,"Rehash") == 0) {
					i->flags.flag_rehash = 1;
					return 0;
				}
				if (strcasecmp(flag,"Shutdown") == 0) {
					i->flags.flag_shutdown = 1;
					return 0;
				}
				if (strcasecmp(flag,"NoFlood") == 0) {
					i->flags.flag_noflood = 1;
					return 0;
				}
				if (strcasecmp(flag,"Modules") == 0) {
					i->flags.flag_modules = 1;
					return 0;
				}
				return -1;
			}
		}
		s = linkedlist_NextItem(s);
	}
	return 0;
}
int ircopinfo_SetPass(const char* opname, const char* pass) {
	void* s = ircoplist_new;
	struct ircopinfo* i;
	while (s != NULL) {
		i = (struct ircopinfo*)linkedlist_GetData(s);
		if (i != NULL) {
			if (strcasecmp(i->name,opname) == 0) {
				if (i->pass != NULL) {return -2;}
				i->pass = malloc(strlen(pass)+1);
				if (i->pass == NULL) {
					return -1;
				}
				strcpy(i->pass,pass);
				return 0;
			}
		}
		s = linkedlist_NextItem(s);
	}
	return -1;
}
int ircopinfo_Add(const char* opname) {
	if (ircoplist_CheckItem(ircoplist_new,opname) >= 0) {
		return -2;
	}
	struct ircopinfo inf;
	memset(&inf,0,sizeof(inf));
	inf.name = malloc(strlen(opname)+1);
	if (inf.name == NULL) {return -1;}
	strcpy(inf.name,opname);
	void* s = linkedlist_Append(&ircoplist_new,&inf,sizeof(inf));
	if (s != NULL) {
		return 0;
	}else{
		free(inf.name);
		return -1;
	}
}
void ircopinfo_ClearNew() {
	ircopinfo_FreeList(ircoplist_new);
	ircoplist_new = NULL;
}
void ircopinfo_Confirm() {
	ircopinfo_FreeList(ircoplist);
	ircoplist = ircoplist_new;
	ircoplist_new = NULL;
	/*void* item = ircoplist;
	struct ircopinfo* i;
	while (item != NULL) {
		i = (struct ircopinfo*)linkedlist_GetData(item);
		if (i != NULL) {
			printf("IRCop %s with %d|%d|%d|%d\n",i->name,i->flags.flag_global,i->flags.flag_services,i->flags.flag_linking,i->flags.flag_sethost);
		}
		item = linkedlist_NextItem(item);
	}*/
}
void reservednicklist_Confirm() {
	linkedlist_Free(&reservednicklist);
	reservednicklist = reservednicklist_new;
	reservednicklist_new = NULL;
}
void noopchans_Confirm() {
	linkedlist_Free(&noopchanlist);
	noopchanlist = noopchanlist_new;
	noopchanlist_new = NULL;
}
int conflisteners_Add(int port, const char* interfaceip, int iptype) {
	void* item = conflisteners_new;
	while (item != NULL) {
		struct conflistener* c = linkedlist_GetData(item);
		if (c->port == port && strcasecmp(c->ip,interfaceip) == 0 && c->iptype == iptype) {
			return -2;
		}
		item = linkedlist_NextItem(item);
	}
	struct conflistener clistener;
	memset(&clistener,0,sizeof(clistener));
	if (interfaceip != NULL) {
		if (strcasecmp(interfaceip,"0.0.0.0") == 0) {
			strcopy(clistener.ip,"any",IPMAXLEN);
		}else{
			strcopy(clistener.ip,interfaceip,IPMAXLEN);
		}
	}
	clistener.port = port;clistener.iptype = iptype;
	void* s = linkedlist_Append(&conflisteners_new,&clistener,sizeof(clistener));
	if (s != NULL) {
		return 0;
	}else{
		return -1;
	}
}
void conflisteners_ClearNew() {
	linkedlist_Free(&conflisteners_new);
	conflisteners_new = NULL;
}
void conflisteners_Confirm() {
	linkedlist_Free(&conflisteners);
	conflisteners = conflisteners_new;
	conflisteners_new = NULL;
}
void listdata_Confirm() {
	reservednicklist_Confirm();
	noopchans_Confirm();
	ircopinfo_Confirm();
	conflisteners_Confirm();
	if (webirclist != NULL) {linkedlist_Free(&webirclist);}webirclist = webirclist_new;webirclist_new = NULL;lastwebirc = NULL;
}
void listdata_ClearNew() {
	reservednicklist_ClearNew();
	noopchans_ClearNew();
	ircopinfo_ClearNew();
	conflisteners_ClearNew();
	if (webirclist_new != NULL) {linkedlist_Free(&webirclist_new);}webirclist_new = NULL;lastwebirc = NULL;
}