//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

int fileopened = 0;
int spacebefore;
int linecomment;
int linepos;
int blockcomment;
int increaseatnextchar = 0;

//getc system vars
int charsread = 0;
int futurechar;
int currchar;

FILE* filedesc;
FILE* creader_FileDesc() {return filedesc;}

int alreadyopened = 0;
void creader_ResetOpened() {
	alreadyopened = 0;
}
int _creader_Init(int writeaccess) {
	//init the reader pre parser's values
	spacebefore = 1;
	fileopened = 1;
	linecomment = 0;blockcomment = 0;
	linepos = 1;
	increaseatnextchar = 0;
	//init the buffered reader (getc system)
	charsread = 0; 
	//the config locations we need to check
	char* errors[5];
#ifndef WIN
	char configureloc[50];
	char seperator[2] = "";
	strcpy(seperator,"/");
	if (strendswith(CONFDIR,"/") == 0) {strcpy(seperator,"");}
	snprintf(configureloc,49,"%s%sweircd.conf",CONFDIR,seperator);configureloc[49] = 0;
	const char* locations[] = {configureloc,"./weircd.conf"};
	unsigned int loccount = 2;
#endif
#ifdef WIN
	const char* locations[] = {"weircdconf.ini"};
	unsigned int loccount = 1;
#endif
	//cycle through all locations and try to open them.
	unsigned int r = 0;
	while (r < loccount) {
		if (writeaccess == 0) {
			filedesc = fopen(locations[r],"r");
		}else{
			filedesc = fopen(locations[r],"w");
		}
		if (filedesc == NULL) {
			//obtain & save details about the error
			const char* verr = tellfileerror();
			errors[r] = malloc(strlen(verr)+1);
			if (errors[r] == NULL) {
				//we cannot allocate space for the error message
				cerr("Failed to allocate error message space",-1);
				//free older errors
				while (r > 0) {
					r--;
					free(errors[r]);
				}
				return -1;
			}
			strcpy(errors[r],verr);
		}else{
			//generate success message
			char successmsg[200];
			snprintf(successmsg,199,"Opening %s",locations[r]);
			successmsg[199] = 0;
			if (alreadyopened == 0) {
				cinfo(successmsg);alreadyopened = 1;
			}
			//throw away old error messages
			while (r > 0) {
				r--;
				free(errors[r]);
			}
			return 0;
		}
		r++;
	}
	//we apparently failed at opening any configs. Spit out errors and quit
	char errormsg[300];
	r = 0;
	while (r < loccount) {
		snprintf(errormsg,299,"Not able to open %s: %s",locations[r],errors[r]);
		errormsg[299] = 0;
		cerr(errormsg,-1);
		free(errors[r]);
		r++;
	}
	return -1;
}
int creader_Init() {
	examinemodule = NULL;
	unloadexaminemodule = 0;
	return _creader_Init(0);
}		
int creader_WriteOpen() {
	return _creader_Init(1);
}

//this is our BUFFERED getc() system, always reading one char into the future to detect and skip comments

int creader_GetCurrC() {
	if (charsread == 0) {
		currchar = fgetc(filedesc);
		if (currchar != EOF) {
			futurechar = fgetc(filedesc);
		}else{
			return CONFEOF;
		}
		charsread = 1;
		return currchar;
	}else{
		currchar = futurechar;
		if (currchar == EOF) {
			return CONFEOF;
		}else{
			futurechar = fgetc(filedesc);
			return (int)currchar;
		}
	}
}
int creader_GetFutureC() {
	if (charsread == 0) {
		creader_GetCurrC();
	}
	if (currchar != EOF) {
		return futurechar;
	}else{
		return CONFEOF;
	}
}
// --> this function returns always the next char in the config file, skipping
//     line and block comments (so only the actually relevant content is returned)
int creader_GetC() {
	if (fileopened == 0) {return CONFEOF;}
	while (1) {
		int readc = creader_GetCurrC();
		if (readc == CONFEOF) {return CONFEOF;}
		if (increaseatnextchar == 1) {linepos++;increaseatnextchar = 0;}
		unsigned char c = (unsigned char)readc;
		if (linecomment == 0 && blockcomment == 0) {
			if (c == ' ' || c == '\r' || c == 9) {
				if (spacebefore == 1) {
					continue;
				}
				if (creader_GetFutureC() != CONFEOF) {
					if ((unsigned char)creader_GetFutureC() == '\n' || (unsigned char)creader_GetFutureC() == ' ' || (unsigned char)creader_GetFutureC() == '\r' || (unsigned char)creader_GetFutureC() == 9 || (unsigned char)creader_GetFutureC() == '=' || (unsigned char)creader_GetFutureC() == ';') {
						//skip as the next one is a line break
						continue;
					}
				}
				spacebefore = 1;
				return ' ';
			}
		}
		if (c == '\n') {
			spacebefore = 1;
			linecomment = 0;
			increaseatnextchar = 1;
			return '\n';
		}
		if (blockcomment == 0 && linecomment == 0) {
			if (c == '/') {
				if (creader_GetFutureC() != CONFEOF) {
					if ((unsigned char)creader_GetFutureC() == '/') {
						linecomment = 1;
						creader_GetCurrC();//drop one char away
						continue;
					}
					if ((unsigned char)creader_GetFutureC() == '*') {
						blockcomment = 1;
						creader_GetCurrC();//drop one char away
						continue;
					}
				}
			}
			if (c == '=' || c == ';') {spacebefore = 1;}else{spacebefore = 0;}
			return c;
		}
		if (blockcomment == 1) {
			if (c == '*') {
				if (creader_GetFutureC() != CONFEOF) {
					if ((unsigned char)creader_GetFutureC() == '/') {
						blockcomment = 0;
						creader_GetCurrC();//drop one char away
						continue;
					}
				}
			}
		}
	}
}
int creader_GetLine() {
	return linepos;
}
void creader_Close() {
	fclose(filedesc);
	if (examinemodule != NULL && unloadexaminemodule == 1) {
		modman_UnloadByPtr(examinemodule);
		examinemodule = NULL;
		unloadexaminemodule = 0;
	}
}