//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2010 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <stdio.h>
#include <stdlib.h>
#include "sockets.h"
#include <string.h>
#include "hostresolver.h"

#if defined(_WIN32) || defined(_WIN64) || defined(WIN32) || defined(WIN64) || defined(__WIN32__) || defined(WINDOWS)
#define WIN
#include <windows.h>
#include <process.h>
#endif

#ifndef WIN
#include <pthread.h>
#endif

#define REQUESTTYPE_REVERSE 1
#define REQUESTTYPE_LOOKUP 2
struct requestinfo {
	int type;
	int wantipv6; //only used for lookups
#ifdef WIN
	//windows threads stuff
	HANDLE threadhandle;
#else
	//pthreads for unix
	pthread_t threadhandle;
	pthread_mutex_t threadeventmutex;
	int threadeventobject;
#endif
	char* requestvalue;
	char* result;
};

#ifdef WIN
void resolvethread(void* data) {
#else
void* resolvethread(void* data) {
#endif
	struct requestinfo* i = data;
	if (i->type == REQUESTTYPE_REVERSE) {
		i->result = malloc(300);
		if (so_ReverseResolveBlocking(i->requestvalue, i->result, 300) != 0) {
			free(i->result);i->result = NULL;
		}
	}
	if (i->type == REQUESTTYPE_LOOKUP) {
		i->result = malloc(IPMAXLEN+1);
		int iptype = IPTYPE_IPV4;
		if (i->wantipv6 == 1) {iptype = IPTYPE_IPV6;}
		if (so_ResolveBlocking(i->requestvalue, iptype, i->result, IPMAXLEN+1) != 0) {
			free(i->result);i->result = NULL;
		}
	}
#ifdef WIN
	return;
#else
	pthread_mutex_lock(&i->threadeventmutex);
	i->threadeventobject = 1;
	pthread_mutex_unlock(&i->threadeventmutex);
	pthread_exit(NULL);
	return NULL;
#endif
}
void* hostresolv_internal_NewRequest(int type, const char* requestvalue, int wantipv6) {
	//do either a reverse lookup or normal lookup request
	struct requestinfo i;
	memset(&i,0,sizeof(i));
	i.requestvalue = malloc(strlen(requestvalue)+1);
	if (!i.requestvalue) {return NULL;}
	strcpy(i.requestvalue, requestvalue);
	i.type = type;
	void* entry = malloc(sizeof(i));
	if (!entry) {free(i.requestvalue);return NULL;}
	memcpy(entry, &i, sizeof(i));
	//now fire off our thread
	struct requestinfo* ri = entry;
	ri->wantipv6 = wantipv6;
#ifdef WIN
	ri->threadhandle = (HANDLE)_beginthreadex(NULL, 0, resolvethread, ri, 0, NULL);
#else
	pthread_mutex_init(&ri->threadeventmutex, NULL);
	ri->threadeventobject = 0;
	pthread_create(&ri->threadhandle, NULL, resolvethread, ri);
	pthread_detach(ri->threadhandle);
#endif
	return entry;
}
void* hostresolv_ReverseLookupRequest(const char* ip) {
	return hostresolv_internal_NewRequest(REQUESTTYPE_REVERSE, ip, 0);
}
void* hostresolv_LookupRequest(const char* host, int ipv6) {
	return hostresolv_internal_NewRequest(REQUESTTYPE_LOOKUP, host, ipv6);
}
int hostresolv_GetRequestStatus(void* handle) {
	if (!handle) {return RESOLVSTATUS_FAILURE;}
	struct requestinfo* i = handle;
#ifdef WIN
	if (WaitForSingleObject(i->threadhandle, 0) == WAIT_OBJECT_0) {
#else
	pthread_mutex_lock(&i->threadeventmutex);
	int u = i->threadeventobject;
	pthread_mutex_unlock(&i->threadeventmutex);
	if (u == 1) {
#endif
		if (i->result != NULL) {
			return RESOLVSTATUS_SUCCESS;
		}else{
			return RESOLVSTATUS_FAILURE;
		}
	}else{
		return RESOLVSTATUS_PENDING;
	}
}
const char* hostresolv_GetRequestResult(void* handle) {
	if (!handle) {return NULL;}
	if (hostresolv_GetRequestStatus(handle) == RESOLVSTATUS_PENDING) {return NULL;}
	struct requestinfo* i = handle;
	return i->result;
}
void hostresolv_FreeRequest(void* handle) {
	if (!handle) {return;}
	struct requestinfo* i = handle;
	free(i->requestvalue);
	if (i->result) {free(i->result);}
#ifdef WIN
	CloseHandle(i->threadhandle);
#else
	
#endif
	free(handle);
}