//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// ----- This is the WeIRCd API for Linking  ----- //
  /* Use this to provide linking functionality.
     This is independent of sockets or the target
     system used, it might also be used to implement
     a twitter bridge or something similar as you
     handle all the connections yourself.
     
     This API isn't fairly trivial. Better check
     the wlinksv1 default implementation for
     example usage.
     
     Please note you need to activate usage of
     this api with weircd_infoq_ReportLinkingFunctionality()
     (see function reference below).

     The following list of functions must be
     implemented by your module as WeIRCd will
     call them (given you activate the linking
     api for your module).
     
  Function needed: int module_ActiveLinkTo(const char* ip, int port, int useipv6);
     This function will be called by WeIRCd when it
     wants your linking module to actively connect
     to another server (the ip is already resolved).
     
     Please note you need to use weircd_api_sockets.h
     (or are at least advised to use it) for connecting
     yourself and, VERY IMPORTANT, your module might
     be asked to maintain multiple connections.
     
     The connectionhandle is an unique connection id
     that WeIRCd generates for you. When you succeed
     to connect but fail to link the server for whatever
     reason, use weircd_linking_LinkTerminated() (see below)
     so the server may generate a subsequent error message.
     
     When succeeding to connect, introduce the server
     with weircd_linking_IntroduceServer().
     
     When the connection to that server
     
     Return 0 on success, -1 on failure.
     
     
  Function needed: int module_PassiveLinkAt(const char* interface, int port, int useipv6, int connectionhandle);
     This function will be called by WeIRCd when it
     wants your linking module to passively listen
     at some port.
     
     Please note you are expected to use e.g.
     weircd_api_sockets.h to manually maintain
     your connection.
     
     The connectionhandle is an unique connection id
     that WeIRCd generates for you.
     
     When a server succeeds to connect to you
     through this passive link interface, use
     weircd_linking_IntroduceServer() to introduce
     it if it links to you successfully.
     
     When a server connects but fails to connect,
     do nothing.

     When a server connects, links successfully
     and then drops connection again/delinks,
     use weircd_linking_SplitServer() on
     that single server.
     
     Please note that you need to manage all
     the connections you get through that
     listening socket yourself. You ARE expected
     to allow multiple links through a single
     listener.
     
     Please note that you may be asked to maintain
     MULTIPLE listeners, each one represented
     by an unique connectionhandle, of which each
     single one may handle multiple incoming server
     links.
     Make sure you are able to handle each listeners
     list of incoming connections separately and that
     you don't accidentially lose a connection!
     
     When module_CloseLink() is called on a passive
     listener connection, you are expected to close
     all the incoming active connections yourself.
     You do *not* need to remove all the servers though,
     WeIRCd will deintroduce them and generate the
     respective netsplit events itself.
     
     Return 0 on success, -1 on failure.
  
  Function needed: void module_CloseLink(int connectionhandle);
     WeIRCd wants you to close the respective listener
     or active connection, no matter if you have
     established a link over it or not.
     
     You do not need to subsequently remove
     all the involved servers. WeIRCd will do that
     for you. Just close your sockets and other
     resources you used for maintaining the internet
     connection.
     
     IMPORTANT: If this is called on your last handled
     connection, CLEAN UP ALL YOUR RESOURCES as your
     module WILL be unloaded.
     
  Function needed: void module_ProceedEvent(const char* servername, int eventid, int eventparametercount);
     This function is called when WeIRCd wants to
     share an event with the server linked through
     that connection
     (or eventually with some indirectly linked
     through that connection, as they will then
     forward it depending on the event).

     Use the module_linking_EventParam...() functions to
     obtain the parameters. (Described below in the
     events section)
     
     Please spread the event to all involved servers.

     If you plan to drop an event, be very careful
     not to desync the servers. You should really know
     what you're doing when just dropping an event.
     
  Function needed: void module_ProceedNewServer(int connectionhandle, const char* name, const char* parent);
     You got a new indirectly linked server. Introduce
     it to your connection. Parent may be NULL with
     a hop count of 1, otherwise the respective parent
     is given (which has already been introduced this
     way).
     
  Function needed: void module_SplitServer(int connectionhandle, const char* name);
     Pass on the split of an indirectly linked server.
     
  Function needed: void module_ProceedNewUser(int connectionhandle, const char* name, const char* servername, const char* userflags);
     Introduce a new user to your link. The server must have been introduced
     previously or will be the NULL for a local user. */

/* (1) Functions to be used during module_InfoQuery() */
  
void weircd_infoq_ReportLinkingFunctionality();
  /* Use this during module_InfoQuery() to activate the linking api for
     your module. */
  
void weircd_infoq_ReportLinkingFunctionalityWithSID();
  /* Use this during module_InfoQuery() to activate the linking api for
     your module and to tell WeIRCd at the same time that you need a proper
     SID (a network-wide unique server identification token composed of a
     single number and two letters, making up a three letter ID) that you
     want to use (e.g. as utilized in TS6 linking). */

/* (2) The event system to share events with the server */

 //  Event types:
 
 #define EVENT_SERVERCONNECTED 1
    // parameters: connection handle [int], name [const char*], sid [const char*], parent [const char*], 
    // sid may be NULL if not using SIDs. parent may be NULL if directly linked (hopcount 1)
 #define EVENT_SERVERDISCONNECTED 2
    // parameters: connection handle [int], sid/servername [const char*]
    // If SIDs are used, only the SID is given, otherwise the name.
 #define EVENT_USERCONNECTED 3
    // parameters: nick [const char*], hopcount [int], lastnickchange [unsigned long], userid [const char*], host [const char*], ip [const char*], uid [const char*], realname [const char*], sid/servername [const char*], usermodes [const char*]
    // If UIDs/SIDs are not used, uid is NULL.
 #define EVENT_USERDISCONNECTED 4
    // parameters: uid/nick [const char*]
    //  Use the following functions to obtain your parameters in your
    //  module_ProceedEvent() function (feeding an invalid param (e.g.
    //  out of range) will result in undefined behaviour)
 #define EVENT_USERCOLLIDED 5
    // parameters: uid/nick [const char*], colliding servername/sid [const char*]
    //  User will be collided and events spread to all other links except
    //  the one where it came from (specified through the servername/sid
    //  of the link's directly connected server)
 #define EVENT_USERRENAMED 6
    // parameters: uid/nick [const char*], new nickname[const char*]
    
  //   Obtain event parameters in module_ProceedEvent():
  
int module_linking_EventParamInt(int param);
const char* module_linking_EventParamStr(int param);
float module_linking_EventParamFloat(int param);

 //  Pass on an event to the server core:
  
void weircd_linking_RaiseEvent(const char* servername, int eventid, int eventparametercount, ...);

/* (3) Informative and miscellaneous functions */
  
void weircd_linking_SetConnectionDataPointer(int connectionhandle, void* ptr);
  /* Use this to set a specific pointer assigned to the connection. You can use
     this to point at a specifically allocated data structure where you keep
     connection-relevant data which you need to manage the connection. */

void* weircd_linking_GetConnectionDataPointer(int connectionhandle);
  /* Obtain the data pointer that is set for the connection. When not
     previously set, it will simply return NULL. */

const char* weircd_linking_GetSID();
  /* Returns a null-terminated string with the SID or NULL if there is none set. */
  
void weircd_linking_CycleConnections_Start();
  /* Start a cycle through all the connection ids that are associcated with
     this module. You may use this e.g. in your select hook to cycle through
     all the connections you're responsible for to check them for new
     data. */

int weircd_linking_CycleConnections_Do();
  /* Execute one step in the cycle. Returns any connection id that is associcated
     with this module. Multiple calls will always give you another one until
     you went through all of them, in which case it will return -1. If you
     wish to cycle through the IDs again, call weircd_linking_CycleConnections_Start()
     to restart the cycle. */
  
int weircd_linking_IsServerPresent(const char* name);
     /* Check whether a specific server is present. When you used an SID to
        introduce it, specify that SID instead of the name.
        Returns 0 if the server is present, otherwise -1. */
  
//int weircd_linking_IntroduceServer(const char* name, const char* sid, const char* description, const char* parentname);
     /* Report a new server. If it has a parent, specify it's name (otherwise NULL).
        If you use SIDs, specify one. You are then expected to specify the SID
        instead of the server's name when addressing it in any other functions.
        Set a description or NULL if you don't have a description.
        
        Parents are used for servers the linked server introduces to you, that is,
        servers with hopcount > 1.
        Returns 0 on success and -1 on failure (which will happen when the server
        has already been introduced previously or if the parent server is not
        known to WeIRCd). */

void weircd_linking_SplitServer(const char* name);
     /* Use this on a server that you have introduced and is now
        gone (also before terminating the link where it has been
        introduced).
        
        You may use it on an active connection aswell, but it is
        useless (simply use weircd_linking_LinkTerminated() and
        WeIRCd will quit all associated servers for you).
        
        It is useful for passive links though as you are then
        responsible for handling multiple connections through
        that interface yourself and where only your module
        can quit the subsequent servers when a single incoming
        connection is gone.
        
        It is also useful on active links when used for servers
        with a hopcount > 2, obviously. Use it to notify those
        indirect splits.
        
        If there are any sub servers to the server specified
        for removal, they will be automatically removed, too.
        
        Local quit events for all affected users will be dis-
        tributed AUTOMATICALLY, you do NOT need to crowd the
        whole net with the quit events.
        
        Please distribute an SQUIT event yourself with
        weircd_linking_Forward():
        
        ":<directly connected server> SQUIT <server to which connection was lost>" */
     
int weircd_linking_LinkTerminated(int connectionhandle);
     /* Report a failure of linking to the given server
        (or also a successful disconnection on request).
        
        The connection handle will become invalid. You
        may also use this when already having introduced
        servers, they will subsequently netsplit.
        
        Don't forget to free the sockets and other
        resources you used on that connection. */

const char* weircd_linking_NickByUID(const char* uid);
     /* Given a user with that UID exists, their nickname will be returned,
        otherwise a NULL pointer. */
     
const char* weircd_linking_GetUID(const char* user);
     /* Return the UID of that user. */

void weircd_linking_RenameUser(const char* name, const char* target);
     /* Rename user. WARNING: Will only work when target nickname is free, so you probably want to check that previously. */

void weircd_linking_Forward(const char* event);
     /* Will send the IRC event to all connections except the
        one that triggered it (by calling the other linking
        module's module_ProceedAndSend() functions). */

int weircd_linking_CheckPassword(int connectionhandle, const char* password, char** sendpassword);
     /* Check a password of a server trying to authenticate
        through the given connection handle. Returns 0 when
        the password is correct, otherwise -1.
        The pointer pointed at by sendpassword will be changed
        to point to a location in memory where a string pointer
        resides that points at a static buffer containing the
        send password to be sent in exchange. This will
        only happen when the function returned 0,
        otherwise the pointer will be left untouched. */

/* Tell us what you're doing! We want to know! */
void weircd_linking_AnnounceLinkConnectAttempt(const char* target);
     /* Announce that you plan to connect to the given target
        (specify e.g. the target ip address). Please also use
        this when some other server connects YOU (passive link)
        to announce that you're about to proceed that server's
        linking request. */
void weircd_linking_AnnounceEstablishedLink(const char* target);
     /* Announce that you connected to the target successfully
        and established a link to it. Also use this for passive
        links that have been established. */
void weircd_linking_AnnounceLinkConnectFail(const char* target, const char* reason);
     /* Annonuce that you falied to establish a link. Possible
        reasons you might specify are: you failed to connect to
        the target server, or the target server uses the
        linking protocol wrong/doesn't support a compatible
        protocol version.
        Please *always* specify a reason. */
void weircd_linking_AnnounceBrokenLink(const char* target, const char* reason);
     /* A link that has been previously announced as being established
        is now broken up again. Specify the reason, e.g. the
        connection timed out or the server violated the
        linking protocol specifications. */
