//    weircdservices module
//    Copyright (C) 2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

/* sqlite database code */

#ifdef PACKAGEDSQLITE
#define SQLITE_THREADSAFE 0
#include "sqlite3.h"
#define SQLITE_THREADSAFE 0
#include "sqlite3.c"
#endif
#ifndef PACKAGEDSQLITE
#include <sqlite3.h>
#endif
sqlite3 *nickdb;
sqlite3 *chandb;
int createversiontables(int initversion);
int setversion(sqlite3* db, int i);
int getversion(sqlite3* db);
int opendbs() {
	char *zErrMsg = 0;
	char dbpath[500];
	//create our databases first
	snprintf(dbpath,499,"%snick.db",weircd_GetDataDir());dbpath[499] = 0;
	int dbp;
	dbp = sqlite3_open(dbpath, &nickdb);
	if (dbp != SQLITE_OK) {
		sqlite3_close(nickdb);
		return -1;
	}
	snprintf(dbpath,499,"%schans.db",weircd_GetDataDir());dbpath[499] = 0;
	dbp = sqlite3_open(dbpath, &chandb);
	if (dbp != SQLITE_OK) {
		sqlite3_close(nickdb);
		sqlite3_close(chandb);
		return -1;
	}
	//create version table with default version when it's not there yet
	if (createversiontables(1) != 0) {
		sqlite3_close(nickdb);sqlite3_close(chandb);
		return -1;
	}
	//create user table in nick.db
	dbp = sqlite3_exec(nickdb,"CREATE TABLE IF NOT EXISTS usertable ( useraccount_name TEXT, useraccount_password TEXT, useraccount_email TEXT, useraccount_registertime INTEGER, userid INTEGER PRIMARY KEY AUTOINCREMENT )",NULL,0,&zErrMsg);
	if (dbp != SQLITE_OK) {
		printf("failed to create nick db table\n");
		sqlite3_free(zErrMsg);
		sqlite3_close(nickdb);
		sqlite3_close(chandb);
		return -1;
	}
	zErrMsg = 0;
	//create channel table and chan flags tables in chan.db
	dbp = sqlite3_exec(chandb,"CREATE TABLE IF NOT EXISTS chantable ( chan_name TEXT, chan_origname TEXT, chan_owner TEXT, chan_topic TEXT, chan_modes TEXT, chan_registertime INTEGER, chan_welcomemsg TEXT, chan_autovoice_activated INTEGER, chanid INTEGER PRIMARY KEY AUTOINCREMENT )",NULL,0,&zErrMsg);
	if (dbp!=SQLITE_OK) {
		printf("failed to create chan db table\n");
		sqlite3_free(zErrMsg);
		sqlite3_close(nickdb);
		sqlite3_close(chandb);
		return -1;
	}
	dbp = sqlite3_exec(chandb,"CREATE TABLE IF NOT EXISTS chanflagstable ( chanid INTEGER, username TEXT, flags TEXT )",NULL,0,&zErrMsg);
	if (dbp!=SQLITE_OK) {
		printf("failed to create chan db flags table\n");
		sqlite3_free(zErrMsg);
		sqlite3_close(nickdb);
		sqlite3_close(chandb);
		return -1;
	}
	return 0;
}
int tableexists(const char* t, sqlite3* db) {
	int k = 0;
	int r = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT name FROM sqlite_master WHERE name=?");
	statement[499] = 0;
	if (sqlite3_prepare_v2(db,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("tableexists: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -2;
	}
	if (sqlite3_bind_text (stmt,1,t,strlen(t),SQLITE_STATIC) != SQLITE_OK) {
		printf("tableexists: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -2;
	}
	if (sqlite3_step(stmt) == SQLITE_ROW) {
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),"name") == 0) {
				const char* v = sqlite3_column_text(stmt,r);
				if (v != NULL) {
					if (strcasecmp(v, t) == 0) {
						if (stmt != NULL) {sqlite3_finalize(stmt);}
						return 0;
					}
				}
			}
			r += 1;
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	return -1;
}
int createvtableondb(sqlite3* db, int initversion) {
	int dbp;char *zErrMsg=0;
	dbp = sqlite3_exec(db,"CREATE TABLE IF NOT EXISTS weircdservicesinfo ( option_name TEXT, option_value TEXT )",NULL,0,&zErrMsg);
	if (dbp!=SQLITE_OK && dbp != SQLITE_DONE) {
		printf("createvtableondb: Could not create table statement.\n");
		sqlite3_free(zErrMsg);
		return -1;
	}
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(db,"INSERT INTO weircdservicesinfo VALUES(?,?)",-1,&stmt,0) != SQLITE_OK) {
		printf("createvtableondb: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	char oname[] = "dbversion";
	char ovalue[10];snprintf(ovalue,9,"%d",initversion);ovalue[9] = 0;
	if (sqlite3_bind_text(stmt,1,oname,strlen(oname),SQLITE_STATIC) != SQLITE_OK) {
		printf("createvtableondb: Could not bind string(0).\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text(stmt,2,ovalue,strlen(ovalue),SQLITE_STATIC) != SQLITE_OK) {
		printf("createvtableondb: Could not bind string(1).\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int result = sqlite3_step(stmt);
	if (result != SQLITE_OK && result != SQLITE_DONE) {
		sqlite3_finalize(stmt);
		printf("createvtableondb: DATABASE ERROR! %d\n",result);
		return -1;
	}
	sqlite3_finalize(stmt);
	return 0;
}
int createversiontables(int initversion) {
	int dbp;
	char *zErrMsg = 0;
	int r = tableexists("weircdservicesinfo",chandb);
	if (r == -2) {return -1;}
	if (r != 0) {
		//we either have a clean database, or a pre-0.7.2 database
		// just in case it's pre-0.72, we need to delete the old,
		// invalid and unused chan flags table so it can be re-
		// created properly
		dbp = sqlite3_exec(chandb,"DROP TABLE chanflagstable",NULL,0,&zErrMsg);
		sqlite3_free(zErrMsg);
		if (createvtableondb(chandb,initversion) < 0) {
			return -1;
		}
	}
	r = tableexists("weircdservicesinfo",nickdb);
	if (r == -2) {return -1;}
	if (r != 0) {
		if (createvtableondb(chandb,initversion) < 0) {
			return -1;
		}
	}
	return 0;
}
int setversion(sqlite3* db, int i) {
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(db,"UPDATE weircdservicesinfo SET option_value=? WHERE option_name=?",-1,&stmt,0) != SQLITE_OK) {
		printf("setversion: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	char valname[] = "dbversion";
	char val[10];snprintf(val,9,"%d",i);val[9] = 0;
	if (sqlite3_bind_text (stmt,1,val,strlen(val),SQLITE_STATIC) != SQLITE_OK) {
		printf("setversion: Could not insert value.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,2,valname,strlen(valname),SQLITE_STATIC) != SQLITE_OK) {
		printf("setversion: Could not insert name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("setversion: failed to update dbversion value (DB error): %d\n",dbp);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	return 0;
}
int getversion(sqlite3* db) {
	int k = 0;
	int r = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT option_value FROM weircdservicesinfo WHERE option_name=?");
	statement[499] = 0;
	if (sqlite3_prepare_v2(db,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("getversion: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -2;
	}
	char value[] = "dbversion";
	if (sqlite3_bind_text (stmt,1,value,strlen(value),SQLITE_STATIC) != SQLITE_OK) {
		printf("getversion: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -2;
	}
	if (sqlite3_step(stmt) == SQLITE_ROW) {
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),"option_value") == 0) {
				const char* v = sqlite3_column_text(stmt,r);
				if (v != NULL) {
					return atoi(v);
				}
			}
			r++;
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	return -1;
}
void closedbs() {
	sqlite3_close(nickdb);
	sqlite3_close(chandb);
}

char* services_getvalue(const char* table, const char* oname, const char* ovalue, const char* target) {
	const char* r1 = NULL;
	char* r2 = NULL;
	unsigned int r,k;r = 0;k = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT * FROM %s WHERE %s=?",table,oname);
	statement[499] = 0;
	sqlite3* dbp = nickdb;
	if (strstw(table,"chan") == 0) {dbp = chandb;}
	if (sqlite3_prepare_v2(dbp,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("getvalue: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return NULL;
	}
	if (sqlite3_bind_text (stmt,1,ovalue,strlen(ovalue),SQLITE_STATIC) != SQLITE_OK) {
		printf("getvalue: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return NULL;
	}
	if (sqlite3_step(stmt) == SQLITE_ROW) {
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),target) == 0) {
				r1 = sqlite3_column_text(stmt,r);
				if (r1 == NULL) {if (stmt != NULL) {sqlite3_finalize(stmt);}return NULL;}
				r2 = malloc(strlen(r1) + 1);
				if (r2 == NULL) {if (stmt != NULL) {sqlite3_finalize(stmt);}return NULL;}
				strcpy(r2,r1);
				if (stmt != NULL) {sqlite3_finalize(stmt);}
				return r2;
			}
			r += 1;
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	r2 = malloc(2);
	if (r2 == NULL) {return NULL;}
	strcpy(r2,"");
	return r2;
}
char* services_getnickvalue(const char* nickname, const char* value) {
	char lclnick[40];
	strncpy(lclnick,nickname,39);lclnick[39] = 0;
	strlwr(lclnick);trimleft(lclnick);cutonfirstspace(lclnick);
	if (strlen(lclnick) <= 0) {return NULL;}
	return services_getvalue("usertable","useraccount_name",lclnick,value);
}
char* services_upassword(const char* nickname) {
	return services_getnickvalue(nickname,"useraccount_password");
}
void services_dropchan(const char* name) {
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(nickdb,"DELETE FROM chantable WHERE chan_name=?",-1,&stmt,0) != SQLITE_OK) {
		printf("DropChan: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,1,name,strlen(name),SQLITE_STATIC) != SQLITE_OK) {
		printf("DropChan: Could not insert name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("DropChan: Failed to delete channel '%s' (DB error)\n",name);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
}
int services_userregistered(const char* nickname) {
	char* r = services_upassword(nickname);
	if (r == NULL) {return 0;}//an error occured, block the nickname until prob is solved
	if (strcasecmp(r,"") == 0) {free(r);return -1;}else{free(r);return 0;}
}
char* services_getemail(const char* nickname) {
	return services_getnickvalue(nickname,"useraccount_email");
}
char* services_getchanvalue(const char* channel, const char* value) {
	char lclchan[40];
	strncpy(lclchan,channel,39);lclchan[39] = 0;
	strlwr(lclchan);trimleft(lclchan);cutonfirstspace(lclchan);
	if (strlen(lclchan) <= 0) {return NULL;}
	return services_getvalue("chantable","chan_name",lclchan,value);
}
char* services_getchannelwelcomemsg(const char* channel) {
	return services_getchanvalue(channel,"chan_welcomemsg");
}
char* services_getchannelorigname(const char* channel) {
	return services_getchanvalue(channel,"chan_origname");
}
char* services_getchannelowner(const char* channel) {
	return services_getchanvalue(channel,"chan_owner");
}
char* services_getchanneltopic(const char* channel) {
	return services_getchanvalue(channel,"chan_topic");
}
void services_setchanneltopic(const char* channel, const char* topic) {
	if (strlen(channel) <= 0 || charpos(channel,' ') > 0 || channel[0] != '#') {return;}
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(chandb,"UPDATE chantable SET chan_topic=? WHERE chan_name=?",-1,&stmt,0) != SQLITE_OK) {
		printf("services_setchanneltopic: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,1,topic,strlen(topic),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_setchanneltopic: Could not insert flags.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,2,channel,strlen(channel),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_setchanneltopic: Could not insert user name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		//printf("services_setchanneltopic: Failed to set new chan topic of %s\n",channel);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
}
int services_channelregistered(const char* channel) {
	char* r = services_getchannelorigname(channel);
	if (r == NULL) {return 0;}//an error occured, block the channel until prob is solved
	if (strcasecmp(r,"") == 0) {free(r);return -1;}else{free(r);return 0;}
}
void services_setnickpass(const char* accname,const char* newpass) {
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(nickdb,"UPDATE usertable SET useraccount_password=? WHERE useraccount_name=?",-1,&stmt,0) != SQLITE_OK) {
		printf("SPass: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,1,accname,strlen(accname),SQLITE_STATIC) != SQLITE_OK) {
		printf("SPass: Could not insert name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,2,newpass,strlen(newpass),SQLITE_STATIC) != SQLITE_OK) {
		printf("SPass: Could not insert pass.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("SPass: Failed to modify the password of user %s (DB error)\n",accname);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
}
void services_chanservjoining() {
	int k = 0;
	int r = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT * FROM chantable");
	statement[499] = 0;
	if (sqlite3_prepare_v2(chandb,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("chanservjoining: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	while (sqlite3_step(stmt) == SQLITE_ROW) {
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),"chan_origname") == 0) {
				const char* v = sqlite3_column_text(stmt,r);
				snprintf(statement,499,"JOIN %s\nMODE %s +o ChanServ",v,v);
				statement[499] = 0;
				weircd_fakeuser_Send(chanserv,statement);
				char* p = services_getchanneltopic(v);
				if (p != NULL) {
					snprintf(statement,499,"TOPIC %s :%s",v,p);
					statement[499] = 0;
					weircd_fakeuser_Send(chanserv,statement);
					free(p);
				}
			}
			r += 1;
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
}
int services_chanid(const char* channel) {
	char lclchan[40];
	strncpy(lclchan,channel,39);lclchan[39] = 0;strlwr(lclchan);trimleft(lclchan);cutonfirstspace(lclchan);
	if (strlen(lclchan) <= 0) {return -1;}
	int k = 0;
	int r = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT * FROM chantable WHERE chan_name=?");
	statement[499] = 0;
	if (sqlite3_prepare_v2(chandb,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("ChanID: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,1,lclchan,strlen(lclchan),SQLITE_STATIC) != SQLITE_OK) {
		printf("ChanID: Could not insert name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_step(stmt) == SQLITE_ROW) {
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),"chanid") == 0) {
				int i = sqlite3_column_int(stmt,r);
				if (stmt != NULL) {sqlite3_finalize(stmt);}
				return i;
			}
			r += 1;
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	return -1;
}
int services_registernick(const char* nickname, const char* pw, const char* mail) {
	char lclnick[40];
	strncpy(lclnick,nickname,39);lclnick[39] = 0;strlwr(lclnick);trimleft(lclnick);cutonfirstspace(lclnick);
	if (strlen(lclnick) <= 0) {return -1;}
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(nickdb,"INSERT INTO usertable VALUES(?,?,?,?,?)",-1,&stmt,0) != SQLITE_OK) {
		printf("RNick: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,1,lclnick,strlen(lclnick),SQLITE_STATIC) != SQLITE_OK) {
		printf("RNick: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,2,pw,strlen(pw),SQLITE_STATIC) != SQLITE_OK) {
		printf("RNick: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,3,mail,strlen(mail),SQLITE_STATIC) != SQLITE_OK) {
		printf("RNick: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_int (stmt,4, weircd_GetCurrentTime()) != SQLITE_OK) {
		printf("RNick: Could not bind int.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int result = sqlite3_step(stmt);
	if (result != SQLITE_OK && result != SQLITE_DONE) {
		sqlite3_finalize(stmt);
		printf("RNick: DATABASE ERROR! %d\n",result);
		return -1;
	}
	sqlite3_finalize(stmt);
	return 0;
}
int services_registerchan(const char* channel, const char* owner, const char* topic) {
	char lclchan[40];
	strncpy(lclchan,channel,39);lclchan[39] = 0;strlwr(lclchan);
	char lclowner[40];
	strncpy(lclowner,owner,39),lclowner[39] = 0;strlwr(lclowner);
	if (strlen(lclchan) <= 0) {return -1;}
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(chandb,"INSERT INTO chantable VALUES(?,?,?,?,?,?,?,?,?)",-1,&stmt,0) != SQLITE_OK) {
		printf("RChan: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,1,lclchan,strlen(lclchan),SQLITE_STATIC) != SQLITE_OK) {
		printf("RChan: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,2,channel,strlen(channel),SQLITE_STATIC) != SQLITE_OK) {
		printf("RChan: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,3,lclowner,strlen(lclowner),SQLITE_STATIC) != SQLITE_OK) {
		printf("RChan: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (topic != NULL) {
		if (sqlite3_bind_text (stmt,4,topic,strlen(topic),SQLITE_STATIC) != SQLITE_OK) {
			printf("RChan: Could not bind string.\n");
			if (stmt != NULL) {sqlite3_finalize(stmt);}
			return -1;
		}
	}
	//leaving 5 (modes)
	if (sqlite3_bind_int (stmt,6,weircd_GetCurrentTime()) != SQLITE_OK) {//topic
		printf("RChan: Could not bind (6).\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	//leaving 7 (welcomemsg) & 8 ((int) autovoice activated & 9 (autoincrement id)
	int result = sqlite3_step(stmt);
	if (result != SQLITE_OK && result != SQLITE_DONE) {
		sqlite3_finalize(stmt);
		printf("RChan: DATABASE ERROR! %d\n",result);
		return -1;
	}
	sqlite3_finalize(stmt);
	return 0;
}
int services_unregisterchan(const char* channel) {
	char lclchan[40];
	strncpy(lclchan,channel,39);lclchan[39] = 0;strlwr(lclchan);
	if (strlen(lclchan) <= 0) {return -1;}
	int chanid = services_chanid(lclchan);
	if (chanid < 0) {printf("DChan: No chan id available.\n");return -1;}
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(chandb,"DELETE FROM chanflagstable WHERE chanid=?",-1,&stmt,0) != SQLITE_OK) {
		printf("DChan(0): Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_int(stmt,1,chanid) != SQLITE_OK) {
		printf("DChan(0): Could not bind int.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int result = sqlite3_step(stmt);
	if (result != SQLITE_OK && result != SQLITE_DONE) {
		sqlite3_finalize(stmt);
		printf("DChan(0): DATABASE ERROR! %d\n",result);
		return -1;
	}
	sqlite3_finalize(stmt);
	stmt = NULL;
	if (sqlite3_prepare_v2(chandb,"DELETE FROM chantable WHERE chan_name=?",-1,&stmt,0) != SQLITE_OK) {
		printf("DChan: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,1,lclchan,strlen(lclchan),SQLITE_STATIC) != SQLITE_OK) {
		printf("DChan: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	result = sqlite3_step(stmt);
	if (result != SQLITE_OK && result != SQLITE_DONE) {
		sqlite3_finalize(stmt);
		printf("DChan: DATABASE ERROR! %d\n",result);
		return -1;
	}
	sqlite3_finalize(stmt);
	return 0;
}
int services_erasechanflags(const char* user, int chanid) {
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(chandb,"DELETE FROM chanflagstable WHERE chanid=? AND username=?",-1,&stmt,0) != SQLITE_OK) {
		printf("services_erasechanflags: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_int(stmt,1,chanid) != SQLITE_OK) {
		printf("services_erasechanflags: Could not bind chan id.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,2,user,strlen(user),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_erasechanflags: Could not bind user name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("services_erasechanflags: Failed to insert flags entry\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	return 0;
}
int services_insertchanflags(const char* user, int chanid, const char* flags) {
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(chandb,"INSERT INTO chanflagstable VALUES(?,?,?)",-1,&stmt,0) != SQLITE_OK) {
		printf("services_insertchanflags: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_int(stmt,1,chanid) != SQLITE_OK) {
		printf("services_insertchanflags: Could not bind chan id.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text(stmt,2,user,strlen(user),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_insertchanflags: Could not bind user name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text(stmt,3,flags,strlen(flags),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_insertchanflags: Could not bind flags.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("services_insertchanflags: Failed to insert flags entry\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	return 0;
}
char chanflagsbuf[100];
const char* services_getchanflags(const char* username, int chanid) {
	char* user = alloca(strlen(username));
	strcpy(user,username);strlwr(user);
	int k = 0;
	int r = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT * FROM chanflagstable WHERE chanid=%d AND username=?",chanid);
	statement[499] = 0;
	if (sqlite3_prepare_v2(chandb,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("services_getchanflags: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return NULL;
	}
	if (sqlite3_bind_text (stmt,1,user,strlen(user),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_getchanflags: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return NULL;
	}
	while (sqlite3_step(stmt) == SQLITE_ROW) {
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),"flags") == 0) {
				const char* v = sqlite3_column_text(stmt,r);
				if (v != NULL) {
					strncpy(chanflagsbuf,v,99);chanflagsbuf[99] = 0;
					if (stmt != NULL) {sqlite3_finalize(stmt);}
					return chanflagsbuf;
				}
			}
			r += 1;
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	strcpy(chanflagsbuf,""); //return a valid empty string when there is no flag entry to be found
	return chanflagsbuf;
}
int services_setchanflags(const char* username, int chanid, const char* flags) {
	char* user = alloca(strlen(username));
	strcpy(user,username);strlwr(user);
	//when the flags are actually none, just remove an old flags entry if present
	if (flags == NULL) {return services_erasechanflags(user,chanid);}
	if (strlen(flags) <= 0) {return services_erasechanflags(user,chanid);}
	//first, check if we have an old flags entry already (if not, we need to insert a new one instead of updating)
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT * FROM chanflagstable WHERE chanid=%d AND username=?",chanid);
	statement[499] = 0;
	if (sqlite3_prepare_v2(chandb,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("services_getchanflags(0): Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		msguser(chanserv,user,"Database error: Statement preparation failed");
		return -1;
	}
	if (sqlite3_bind_text (stmt,1,user,strlen(user),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_getchanflags(0): Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE && dbp != SQLITE_ROW) {
		printf("services_setchanflags(0): Failed to check for present flags entry: %d\n",dbp);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (dbp != SQLITE_ROW) {
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return services_insertchanflags(user,chanid,flags);
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	//ok, apparently there is an older entry we want to update with the new values:
	stmt = NULL;
	if (sqlite3_prepare_v2(chandb,"UPDATE chanflagstable SET flags=? WHERE username=? AND chanid=?",-1,&stmt,0) != SQLITE_OK) {
		printf("services_setchanflags: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,1,flags,strlen(flags),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_setchanflags: Could not insert flags.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,2,user,strlen(user),SQLITE_STATIC) != SQLITE_OK) {
		printf("services_setchanflags: Could not insert user name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_int(stmt,3,chanid) != SQLITE_OK) {
		printf("services_setchanflags: Could not insert chanid.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("services_setchanflags: Failed to modify the chan flags of user %s (DB error)\n",user);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	return 0;
}