//    weircdservices module
//    Copyright (C) 2009-2010 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "../weircd_api.h"
#include "../weircd_api_linking.h"
#include "../weircd_api_sockets.h"

#define READBUF 2048
#define SENDBUF (4096*10)

struct subconninfo {
	char* receivedpass;
	char* sid;
	unsigned long long time;
	char* readbuf;
	char* sendbuf;
	char* capabs; //received CAPAB parameters of the server
	char* ip; // ip of the target server
	int syncstate; //0 - none; 1 - server and burst, 2 - burst done
	int socket; // the same as the main connection info's socket for active connections
};

struct connectioninfo {
	int ispassive;
	int socket;
	int iptype;
	void* listofsubconns;
};

void freesubconndata(void* ptr) {
	struct subconninfo* si = linkedlist_GetData(ptr);
	if (si->receivedpass != NULL) {free(si->receivedpass);}
	if (si->sid != NULL) {free(si->sid);}
	if (si->capabs != NULL) {free(si->capabs);}
	if (si->readbuf != NULL) {free(si->readbuf);}
	if (si->sendbuf != NULL) {free(si->sendbuf);}
	if (si->ip != NULL) {free(si->ip);}
}

int major = 1;
int minor = 2;
int release = 0;
void module_InfoQuery() {
	weircd_infoq_ReportName("wlinksv1 Linking");
	weircd_infoq_ReportVersion(major,minor,release);
	weircd_infoq_ReportComment("A linking module for linking multiple WeIRCd instances with TS6 (non-cyclic)");
	weircd_infoq_ReportAuthor("Jonas Thiem");
	weircd_infoq_ReportLinkingFunctionalityWithSID();
}
int module_PassiveLinkAt(const char* interface, int port, int useipv6, int connectionhandle) {
	struct connectioninfo c;
	memset(&c,0,sizeof(c));
	c.ispassive = 1;
	int iptype = IPTYPE_IPV4;if (useipv6 == 1) {iptype = IPTYPE_IPV6;}
	c.socket = weircd_sockets_CreateSocket(1, iptype);
	c.iptype = iptype;
	if (c.socket < 0) {return -1;}
	void* ptr = malloc(sizeof(struct connectioninfo));
	if (ptr == NULL) {weircd_sockets_CloseSocket(c.socket);return -1;}
	memcpy(ptr, &c, sizeof(c));
	weircd_linking_SetConnectionDataPointer(connectionhandle,ptr);
	weircd_sockets_HookSelect();
	/* now open our listener */
	if (weircd_sockets_MakeSocketListen(c.socket, port, iptype, interface) != 0) {
		free(ptr);
		return -1;
	}
	return 0;
}
void closeconn(int handle, void* subconnptr) { // WILL close main socket of active links aswell
	freesubconndata(subconnptr);
	void* ptr = weircd_linking_GetConnectionDataPointer(handle);
	if (ptr == NULL) {return;}
	struct subconninfo* si = linkedlist_GetData(subconnptr);
	struct connectioninfo* c = (struct connectioninfo*)ptr;
	if (si->syncstate == 1) {
		/* we introduced the server already. ergo: we need to remove it again! */
		weircd_linking_SplitServer(si->sid);
	}
	if (c->ispassive == 0) {
		/* Active connection with no other sub connections.
		   Therefore, also close the active link itself. */
		weircd_sockets_CloseSocket(c->socket);
		free(c);
		//weircd_linking_LinkTerminated(handle);
	}else{
		/* Passive connection. Just close the socket of that
		   connection. */
		weircd_sockets_CloseSocket(si->socket);
	}
	/* Remove the sub connection item of the list. Will also free the struct itself */
	linkedlist_RemoveItem(&c->listofsubconns, subconnptr);
}
void sendonlink(const char* line, int handle, void* subconnptr) {
	if (strlen(line) <= 0) {return;}
	printf("SENDING THIES: %s\n",line);
	/* add something to that link's sendbuffer */
	struct subconninfo* si = linkedlist_GetData(subconnptr);
	if (si->sendbuf == NULL) {
		si->sendbuf = malloc(SENDBUF);
		if (si->sendbuf == NULL) {return;}//no idea what to do...
		si->sendbuf[0] = 0;
	}
	if (strlen(si->sendbuf) + strlen(line) + 1 + 1 < SENDBUF) {
		if (strlen(si->sendbuf) <= 0) {
			weircd_sockets_IncreaseSend();
		}
		strcat(si->sendbuf,line);strcat(si->sendbuf,"\n");
	}
}
char response[1024];
// Definitions and vars for splitting up the parameters
  //maximum amount of parameters;
#define MAXPARAMS 15 
  //maximum len of a normal parameter:
#define MAXCMDLEN 40
  //maximum len of the long last parameter:
#define MAXLASTCMDLEN 700
  //static buffer array for the splitted up normal parameters:
char** cmdbuf = NULL;
  //static buffer array for the pointers to the parameters
char** argv = NULL;
  //static buffer for the long last parameter:
char lastcmdbuf[MAXLASTCMDLEN];
int processline(const char* line, int handle, void* subconnptr) {
	char* l = alloca(strlen(line)+1);
	strcpy(l,line);trimleft(l);trimright(l);
	struct subconninfo* sc = linkedlist_GetData(subconnptr);
	char prefix[50] = "";
	if (strstw(l,":") == 0) {//we got a prefix! extract it
		cut(l,1);
		strcopy(prefix,l,49);
		cutonfirstspace(prefix);
		if (charpos(l,' ') > 0) {cut(l,charpos(l,' ')+1);trimleft(l);}else{strcpy(l,"");}
	}
	if (strstw(l,"PASS ") == 0) {
		if (sc->receivedpass != NULL) {
			//what do we do? well... better quit
			closeconn(handle, subconnptr);
			return -1;
		}else{
			cut(l,strlen("PASS "));
			if (charpos(l,' ') <= 0) {
				weircd_linking_AnnounceLinkConnectFail(sc->ip, "Invalid PASS command");
				closeconn(handle, subconnptr);
				return -1;
			}
			char pass[50];
			strcopy(pass,l,49);cutonfirstspace(pass);
			cut(l,charpos(l,' ')+1);
			trimleft(l);
			/* It needs to be TS of course! */
			if (strstw(l, "TS ") != 0) {
				weircd_linking_AnnounceLinkConnectFail(sc->ip, "Invalid PASS command");
				closeconn(handle, subconnptr);
				return -1;
			}
			cut(l,3);trimleft(l);
			/* Now check version. We only support TS6 so far */
			if (strstw(l,"6 ") != 0) {
				weircd_linking_AnnounceLinkConnectFail(sc->ip, "Unsupported TS version");
				closeconn(handle, subconnptr);
				return -1;
			}
			cut(l,2);trimleft(l);
			if (strstw(l,":") == 0) {cut(l,1);trimleft(l);}
			if (strlen(l) != 3) {
				closeconn(handle, subconnptr);return -1;
			}
			//ok, check the server ID
			int validid = 0;
			if (l[0] < '0' || l[1] > '9') {
				validid = -1;
			}
			if ((l[0] < '0' || l[1] > '9') && (l[1] < 'a' || l[1] > 'z') && (l[1] < 'A' || l[1] > 'Z')) {validid = -1;}
			if ((l[0] < '0' || l[1] > '9') && (l[2] < 'a' || l[2] > 'z') && (l[2] < 'A' || l[2] > 'Z')) {validid = -1;}
			if (validid < 0) {
				weircd_linking_AnnounceLinkConnectFail(sc->ip, "Invalid SID specified");
				closeconn(handle, subconnptr);return -1;
			}
			sc->sid = malloc(strlen(l)+1);
			if (sc->sid == NULL) {
				weircd_linking_AnnounceLinkConnectFail(sc->ip, "malloc() returned NULL");
				closeconn(handle, subconnptr);return -1;
			}
			strcpy(sc->sid, l);
			sc->receivedpass = malloc(strlen(pass)+1);
			if (sc->receivedpass == NULL) {
				weircd_linking_AnnounceLinkConnectFail(sc->ip, "malloc() returned NULL");
				closeconn(handle, subconnptr);return -1;
			}
			strcpy(sc->receivedpass,pass);
			return 0;
		}
	}
	if (strstw(l,"CAPAB ") == 0) {
		if (sc->syncstate > 0) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "Unexpected CAPAB command");
			closeconn(handle, subconnptr);return -1;
		}
		if (sc->receivedpass == NULL) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "Unexpected CAPAB command");
			closeconn(handle, subconnptr);return -1; // we WANT the password first.
		}
		if (sc->capabs != NULL) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "CAPAB used twice");
			closeconn(handle, subconnptr);return -1; // twice? Nooo
		}
		cut(l,strlen("capab "));trimleft(l);
		if (strstw(l,":") == 0) {cut(l,1);trimleft(l);}
		sc->capabs = malloc(strlen(l)+1);
		if (sc->capabs == NULL) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "malloc() returned NULL");
			closeconn(handle, subconnptr);return -1; // malloc fail
		}
		strcpy(sc->capabs,l);
		return 0;
	}
	if (strstw(l,"SERVER ") == 0) {
		if (sc->syncstate > 0) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "Unexpected SERVER command");
			closeconn(handle, subconnptr);return -1;
		}
		cut(l,strlen("server "));
		trimleft(l);
		if (charpos(l,':') <= 0) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "Invalid SERVER command");
			closeconn(handle, subconnptr);return -1;
		}
		char sname[70];
		strcopy(sname, l, 69);
		cutonfirstspace(sname);
		cut(l,charpos(l,':')+1);trimleft(l);
		if (strlen(l) <= 0 || strlen(sname) < 2) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "Invalid SERVER command");
			closeconn(handle, subconnptr);return -1;
		}
		
		//ok. first, check the password
		char* pwwesend = NULL;
		if (weircd_linking_CheckPassword(handle, sc->receivedpass, &pwwesend) != 0) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "Wrong password");
			closeconn(handle, subconnptr);return -1;
		}
		//server is ok. accept it
		if (weircd_linking_IntroduceServer(sname, sc->sid, l, NULL) < 0) {
			weircd_linking_AnnounceLinkConnectFail(sc->ip, "Server already known");
			closeconn(handle, subconnptr);return -1;
		}
		sc->syncstate = 1;
		weircd_linking_AnnounceEstablishedLink(sc->ip);
		//then we need to send our stuff.
		printf("OUR PW IS: '%s'\n",pwwesend);
		snprintf(response,1023,"PASS %s TS 6 :%s",pwwesend,weircd_linking_GetSID());
		sendonlink(response,handle,subconnptr);
		sendonlink("CAPAB :QS GLN HOPS HUB ENCAP",handle,subconnptr);
		snprintf(response,1023,"SERVER %s 1 :%s",weircd_GetServerName(),weircd_GetServerDescription());
		sendonlink(response,handle,subconnptr);
		snprintf(response,1023,":%s SVINFO 6 6 0 %llu",weircd_linking_GetSID(),weircd_GetCurrentTime());
		sendonlink(response,handle,subconnptr);
		//that should be enough to get us logged in!
		sc->syncstate = 1;
		//send all user data NAOW
                /* (1) Burst servers* /
                
                /* (2) Burst user data */
		weircd_user_CycleAll_Init(1);
		const char* p = weircd_user_CycleAll_Do();
		while (p != NULL) {
                        char sid[4];
                        const char* p = weircd_user_GetUID(p);
                        if (p != NULL) {
                                strcopy(sid,p,3);
                        }else{
                                p = weircd_user_CycleAll_Do();
                                continue;
                        }
                        snprintf(response,1023,":%s UID %s %d %lu w%s %s %s %s %s :%s",weircd_linking_GetSID(),p, weircd_user_GetHopcount(p), weircd_user_GetNickTS(p), weircd_user_GetUmodes(p), weircd_user_GetUserid(p), weircd_user_GetHost(p), weircd_user_GetIp(p), weircd_linking_GetUID(p), weircd_user_GetRealname(p));
                        sendonlink(response,handle,subconnptr);
                        p = weircd_user_CycleAll_Do();
                }
                /* (3) Burst channel data */
                
		return 0;
	}
	if (sc->syncstate != 1) {
		weircd_linking_AnnounceLinkConnectFail(sc->ip, "Invalid command during login sequence");
		closeconn(handle, subconnptr);return -1;
	}
	if (strlen(prefix) <= 0) {
		strcopy(prefix, sc->sid,49);
	}
	int argc;
	if (cmdbuf == NULL) {
		obtainsplitargsbuf(&cmdbuf, MAXPARAMS, MAXCMDLEN);
	}
	if (argv == NULL) {
		obtainptrbuf(&argv, MAXPARAMS);
	}
	splitcmdargs(l, MAXPARAMS, MAXCMDLEN, MAXLASTCMDLEN, cmdbuf, lastcmdbuf, &argc, argv);
	if (argc <= 0) {
		//no command. hum.
		return 0;
	}
	if (strcasecmp(argv[0],"ping") == 0) {
		if (argc <= 1) {
			//no parameters?
			return 0;
		}
		snprintf(response,1024,"PONG :%s", argv[1]);
		sendonlink(response,handle,subconnptr);
		return 0;
	}
	if (strcasecmp(argv[0],"uid") == 0) {
		//ok. let's check whether the SID is known to us:
		if (weircd_linking_IsServerPresent(prefix) != 0) {
			weircd_linking_AnnounceBrokenLink(sc->ip,"Invalid SID for new user");
			closeconn(handle, subconnptr);return -1;
		}
		if (argc < 9 || argc > 10) { //8 or 9 arguments (substract command itself); 9th is optional real name
			weircd_linking_AnnounceBrokenLink(sc->ip,"Unsuitable argument count for UID command");
			closeconn(handle, subconnptr);return -1;
		}
		if (weircd_user_IsOn(argv[1]) == 0) { // COLLISION !!
			//compare timestamps
			if (atol(argv[3]) < weircd_user_GetNickTS(argv[1])) {
				// received timestamp is OLDER
				// check whether user@ip are identical (we are _not_ checking user@host
				// as a host may be more easily manipulated on WeIRCd servers)
				if (strcasecmp(weircd_user_GetUserid(argv[1]), argv[5]) == 0
				&& strcasecmp(weircd_user_GetIp(argv[1]), argv[7]) == 0) {
					//they are! kill off older remote user (allow changing servers more easily)
					snprintf(response,1024,"KILL %s :Killed by %s: \"Nick collision\"", argv[8], weircd_GetServerName());
					sendonlink(response,handle,subconnptr);
					return; // do not continue and add new user locally
				}else{
					//they are not! collide newer existing user (attack prevention for older users)
					weircd_linking_CollideUser(argv[1]);
				}
				
			}else {
				// received timestamp is NEWER
				// check whether user@ip are identical
				if (strcasecmp(weircd_user_GetUserid(argv[1]), argv[5]) == 0
				&& strcasecmp(weircd_user_GetIp(argv[1]), argv[7]) == 0) {
					//they are! collide older existing user (allow changing servers more easily)
					weircd_linking_CollideUser(argv[1]);
				}else{
					//they are not! kill off newer remote user (attack prevention for older users)
					snprintf(response,1024,"KILL %s :Killed by %s: \"Nick collision\"", argv[8], weircd_GetServerName());
					sendonlink(response,handle,subconnptr);
					return; // do not continue and add new user locally
				}
			}
		}
		char* realname = NULL;
		if (argc == 10) {
			realname = argv[9];
		}
		weircd_linking_IntroduceUser(argv[1], atoi(argv[2]), strtoul(argv[3],NULL,0), argv[5],
		argv[6], argv[7], argv[8], realname, prefix, argv[4]);
		//now we still need to forward this to other servers
                /*snprintf(response,1023,":%s!%s@%s CONNECT :%s",p,argv[1]);
                weircd_linking_Forward(response);*/
		return;
	}
	if (strcasecmp(argv[0],"sid") == 0) {
                //ok. let's check whether the SID is known to us:
                if (weircd_linking_IsServerPresent(prefix) != 0) {
                        weircd_linking_AnnounceBrokenLink(sc->ip,"Invalid SID parent for new server");
                        closeconn(handle, subconnptr);return -1;
                }
                if (argc != 5) { // 4 arguments + command itself
                        weircd_linking_AnnounceBrokenLink(sc->ip,"Unsuitable argument count for SID command");
                        closeconn(handle, subconnptr);return -1;
                }
                if (strcasecmp(prefix,weircd_linking_GetSID()) == 0) {
                        //directly linked to the connected server
                        weircd_linking_IntroduceServer(argv[1],argv[3],argv[4],NULL);
                }else{
                        //has another parent
                        weircd_linking_IntroduceServer(argv[1],argv[3],argv[4],prefix);
                }
                return;
        }
        if (strcasecmp(argv[0],"nick") == 0) {
                //ok. let's check whether the SID is known to us:
                if (strlen(prefix) != 3+5) {
                        weircd_linking_AnnounceBrokenLink(sc->ip,"Invalid UID for NICK command");
                        closeconn(handle, subconnptr);return -1;
                }
                if (argc != 3) { // 2 arguments + command itself
                        weircd_linking_AnnounceBrokenLink(sc->ip,"Unsuitable argument count for SID command");
                        closeconn(handle, subconnptr);return -1;
                }
                const char* nickofuser = weircd_linking_NickByUID(prefix);
                if (nickofuser == NULL) {
                        //unknown user. ignore in case of desync
                        return 0;
                }
                if (weircd_user_IsOn(argv[1]) == 0) {
                        // COLLISION !
                        
                }
                // rename user
                weircd_linking_RenameUser(nickofuser, argv[1]);
                // forward the rename event to others (?)
                snprintf(response,1023,":%s NICK :%s",prefix,argv[1]);
                weircd_linking_Forward(response);
                return;
        }
	return 0;
}
char staticbuf[READBUF];
char* ip = NULL; //store ip here temporarily
void module_SelectHook(int readdata, int writedata) {
	//printf("Yea, hooked. READ: %d WRITE: %d\n",readdata,writedata);
	weircd_linking_CycleConnections_Start();
	int i = weircd_linking_CycleConnections_Do();
	while (i >= 0) {
		struct connectioninfo* c = weircd_linking_GetConnectionDataPointer(i);
		if (readdata == 1) {
			/* check for data to be read: */
			void* item = c->listofsubconns;
			while (item != NULL) {
				struct subconninfo* sc = linkedlist_GetData(item);
				void* nextitem = linkedlist_NextItem(item);
				if (weircd_sockets_SelectSaysRead(sc->socket) == 0) {
					/*YAY read*/
					int len = strlen(sc->readbuf);
					if (len+1 >= READBUF) {strcpy(sc->readbuf,"");len = 0;}
				
					int z = so_ReceiveData(sc->socket,sc->readbuf+len, READBUF - (len+1));
					if (z > 0) {//we received something!
						sc->readbuf[len+z] = 0;
						fixlinebreaks(sc->readbuf,strlen(sc->readbuf));
						int cpos = charpos(sc->readbuf,'\n');
						while (cpos >= 0) {
							strcpy(staticbuf,sc->readbuf);
							staticbuf[cpos] = 0;
							cut(sc->readbuf,cpos+1);
							printf("GOT A LINE: %s\n",staticbuf);
							if (processline(staticbuf, i, item) < 0) {
								break;
							}
							cpos = charpos(sc->readbuf,'\n');
						}
					}
				}
				item = nextitem;
			}
		}
		if (c->ispassive == 1 && readdata == 1) {
			/* ok. check foar connections naow */
			if (ip == NULL) {
				ip = malloc(weircd_sockets_GetIPLen()+1);
			}
			int sock;
			if (ip == NULL) {break;} //no sense in going on
			if (weircd_sockets_AcceptConnection(c->socket, c->iptype, ip, &sock) == 0) {
				weircd_linking_AnnounceLinkConnectAttempt(ip);
				struct subconninfo s;memset(&s,0,sizeof(s));
				s.readbuf = malloc(READBUF);
				if (s.readbuf == NULL) {
					close(sock);
					free(ip);
					return; //no point in going on
				}
				s.readbuf[0] = 0;
				s.ip = ip;
				s.time = weircd_GetCurrentTime();
				s.socket = sock;
				linkedlist_Append(&c->listofsubconns,&s,sizeof(s));
				ip = malloc(weircd_sockets_GetIPLen()+1); //get us a new buffer
			}
		}
		if (writedata == 1) {
			/* check for data to be sent: */
			void* item = c->listofsubconns;
			while (item != NULL) {
				struct subconninfo* sc = linkedlist_GetData(item);
				if (sc->sendbuf != NULL) {
					if (weircd_sockets_SelectSaysWrite(sc->socket) == 0 && strlen(sc->sendbuf) > 0) {
						int r = weircd_sockets_SendOnSocket(sc->socket,sc->sendbuf,strlen(sc->sendbuf));
						if (r <= 0) {
							if (r == 0) { //eof. omg. what do we do.
							
							}
						}else{
							if (r >= strlen(sc->sendbuf)) {
								weircd_sockets_DecreaseSend();
							}
							cut(sc->sendbuf,r);
						}
					}
				}
				item = linkedlist_NextItem(item);
			}
		}
		i = weircd_linking_CycleConnections_Do();
	}
}