//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

int usinguids = 0; //we are not using them by default. when they're needed by a linking module, they will be
                   //bulk generated and in the future generated and assigned to every new user instantly
int uids_CheckIfFree(const char* uid) {
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (udat(u)->uid != NULL && udat(u)->pleaseremove == 0) {
			if (strcasecmp(udat(u)->uid,uid) == 0) {
				return -1;
			}
		}
		u = usr_DoCycle();
	}
	return 0;
}
#define UIDLEN 6
int uid[UIDLEN];
int uidcounterset = -1;
char uidbuf[UIDLEN+1];
char combineduid[UIDLEN+3+1];
const char* uids_Get() {
	int k = 0;
	while (k < UIDLEN) {
		uidbuf[k] = uid[k]+'A';
		k++;
	}
	uidbuf[UIDLEN] = 0;
	//now increase so the next user will get something else
	uid[0]++;
	//check whether we've overflown a digit and increase the next
	int recheck = 1;
	while (recheck == 1) {
		recheck = 0;
		int r = 0;
		while (r < UIDLEN) {
			if (uid[r] >= 26) {
				//go back to zero
				uid[r] = 0;
				if (r < UIDLEN) {
					//increase next digit
					uid[r+1]++;
					recheck = 1;
					break;
				}else{
					//we reached the end of all possible combinations. start from the beginning
					r = 0;
					while (r < UIDLEN) {
						uid[r] = 0;
						r++;
					}
					break;
				}
			}
			r++;
		}
	}
	return uidbuf;
}
void uids_Assign(USER u) { // WARNING: will destroy your current usr_DoCycle()
	if (uidcounterset < 0) {
		int r = 0;
		while (r < UIDLEN) {
			uid[r] = 0;
			r++;
		}
		uidcounterset = 1;
	}
	const char* id = uids_Get();
	while (uids_CheckIfFree(id) != 0) {id = uids_Get();}
	udat(u)->uid = malloc(UIDLEN+3+1);
	if (udat(u)->uid == NULL) {return;}
	const char* sid = NULL;
	if (udat(u)->homeserver != NOSERVER) {
                sid = server_GetName(udat(u)->homeserver);
                if (strlen(sid) != 3) {
                        sid = NULL;
                }
        }else{
                sid = config_StrVal("SID");
                if (strlen(sid) != 3) {
                        sid = NULL;
                }
        }
        if (sid != NULL) {
                strcpy(udat(u)->uid, sid);
        }else{
                strcpy(udat(u)->uid,"");
        }
	strcat(udat(u)->uid, id);
}
void uids_Activate() {
	if (usinguids == 1) {return;}
	usr_CycleInit();
	USER u = usr_DoCycle();
	//ok. cycling here is unsafe. uids_Assign() *will* destroy our cycle. so let's advance through the list manually
	while (u != NOUSER) {
		if (udat(u)->uid == NULL && udat(u)->pleaseremove == 0) {
			uids_Assign(u);
		}
		u = linkedlist_NextItem(u);
	}
}