//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

// 1 -- User Array Management

void* linkeduserlist = NULL;
struct weircd_user* udat(USER usr) {
	return linkedlist_GetData((void*)usr);
}
USER usr_FindByNick(const char* nick) {
	void* item = linkeduserlist;
	while (item != NULL) {
		struct weircd_user* d = (struct weircd_user*)linkedlist_GetData(item);
		if (d != NULL) {
			if (strcasecmp(d->nickname,nick) == 0) {
				return item;
			}
		}
		item = linkedlist_NextItem(item);
	}
	return NOUSER;
}
USER usr_AddUser(int socket, int mastersocket, const char* ip, int iptype) {
	struct weircd_user dat;
	memset(&dat,0,sizeof(dat));
	if (socket >= 0) {
		dat.socket = socket;
		dat.mastersocketid = mastersocket;
		dat.CONNECTtime = ts();
		dat.iptype = iptype;
		if (charpos(ip,':') > 0) {
			strcpy(dat.ip,"[");
			stradd(dat.ip,ip,IPMAXLEN);
			stradd(dat.ip,"]",IPMAXLEN);
		}else{
			strcpy(dat.ip,ip);
		}
	}else{
		dat.fakeuser = 1;
		dat.socket = -1;
		dat.mastersocketid = -1;
#ifdef IPV6
		strcpy(dat.ip,"[::1]");
		dat.iptype = IPTYPE_IPV6;
#endif
#ifndef IPV6
		strcpy(dat.ip,"127.0.0.1");
		dat.iptype = IPTYPE_IPV4;
#endif
	}
	void* retval = linkedlist_Prepend(&linkeduserlist, &dat, sizeof(dat));
	if (usinguids == 1) {
		uids_Assign(retval);
	}
	return retval;
}
void usr_DelUser(USER usr) {
	usr_chans_Clear(usr);
	int r = 0;
	/* free send buffers */
	while (r < USER_MAXSENDBUFFERS) {
		if (udat(usr)->sendbuffer[r] != NULL) {
			free(udat(usr)->sendbuffer[r]);
		}
		r++;
	}
	/* free channel info data */
	linkedlist_Free(&udat(usr)->channelinfolist);
	/* free some other data */
	if (udat(usr)->awayreason != NULL) {free(udat(usr)->awayreason);}
	if (udat(usr)->quitreason != NULL) {free(udat(usr)->quitreason);}
	if (udat(usr)->servname != NULL) {free(udat(usr)->servname);}
	if (udat(usr)->accountname != NULL) {free(udat(usr)->accountname);}
	if (udat(usr)->resolvedhost != NULL) {free(udat(usr)->resolvedhost);}
	/* remove us from the sending count */
	wantsend -= udat(usr)->sendblocks;
	/* remove all invites pointing to that user */
	chan_CycleInit();
	CHANNEL chan = chan_DoCycle();
	while (chan != NOCHAN) {
		int r = 0;
		while (r < MAXINVITESPERCHAN) {
			if (cdat(chan)->invite[r] != NULL) {
				if (cdat(chan)->invite[r]->target == usr) {
					chan_FreeInvite(chan, r);
					cdat(chan)->invite[r] = 0;
				}
			}
			r++;
		}
		chan = chan_DoCycle();
	}
	/* now remove the linked list entry */
	linkedlist_RemoveItem(&linkeduserlist,(void*)usr);
}
USER cycleuser;
void usr_CycleInit() {
	cycleuser = linkeduserlist;
}
USER usr_DoCycle() {
	if (cycleuser == NULL) {return NULL;}
	void* retval = cycleuser;
	cycleuser = linkedlist_NextItem(cycleuser);
	return retval;
}
int usr_SlotCount() {
	int count = 0;
	void* item = linkeduserlist;
	while (item != NULL) {
		count++;
		item = linkedlist_NextItem(item);
	}
	return count;
}
void usr_CloseAllConnectionsForShutdown() {
	void* u = linkeduserlist;
	while (u != NULL) {
		if (udat(u)->socket >= 0 && udat(u)->hopcount <= 0 && udat(u)->fakeuser == 0) {
			close(udat(u)->socket);
			udat(u)->socket = -1;
		}
		u = linkedlist_NextItem(u);
	}
}

// 2 - handling the channel list of a user

void* cycleusrchan;
void usr_chans_CycleInit(USER uid) {
	cycleusrchan = udat(uid)->channelinfolist;
}
void* usr_chans_DoCycle() {
	if (cycleusrchan == NULL) {return NULL;}
	void* retval = cycleusrchan;
	cycleusrchan = linkedlist_NextItem(cycleusrchan);
	return retval;
}
void usr_chans_Clear(USER uid) {
	linkedlist_Free(&udat(uid)->channelinfolist);
	udat(uid)->channelinfolist = NULL;
}
struct weircd_user_chaninfo* usr_chans_GetInfo(void* item) {
	return linkedlist_GetData(item);
}
void usr_chans_Delete(USER usr, void* item) {
	linkedlist_RemoveItem(&udat(usr)->channelinfolist,item);
}
void* usr_chans_Add(USER uid) {
	struct weircd_user_chaninfo cinfo;
	memset(&cinfo,0,sizeof(cinfo));
	cinfo.restrictedcache = -1;
	return linkedlist_Append(&udat(uid)->channelinfolist,&cinfo,sizeof(cinfo));
}
int usr_chans_Count(USER uid) {
	void* item = udat(uid)->channelinfolist;
	int count = 0;
	while (item != NULL) {
		count++;
		item = linkedlist_NextItem(item);
	}
	return count;
}
void usr_chans_Print(USER uid) {//for debugging
	void* item = udat(uid)->channelinfolist;
	while (item != NULL) {
		struct weircd_user_chaninfo* c = linkedlist_GetData(item);
		printf("inchan %s ",cdat(c->cptr)->name);
		item = linkedlist_NextItem(item);
	}
	printf("\n");
	return;
}
void* usr_chans_GetHandleByChan(USER uid, CHANNEL chan) {
	void* item = udat(uid)->channelinfolist;
	while (item != NULL) {
		struct weircd_user_chaninfo* cinfo = linkedlist_GetData(item);
		if (cinfo->cptr == chan) {
			return item;
		}
		item = linkedlist_NextItem(item);
	}
	return NULL;
}


// 3 - other user handling functions

void usr_SendConnectionNotice(const char* nick, const char* sid, const char* host, const char* ip) {
	
	void* item = linkeduserlist;
	while (item != NULL) {
		struct weircd_user* u = linkedlist_GetData(item);
		if (u->pleaseremove == 0 && u->authed == 1) {
			if (u->mode_servicesbot == 1) {
				usnd(item,":%s 650 %s %s",svname(),u->nickname,nick);
			}else{
				if (u->mode_connectionnotices == 1 && sid != NULL && host != NULL && ip != NULL) {
					usnd(item,":%s NOTICE %s :*** Notice -- Client connecting: %s (%s@%s) [%s]",svname(),u->nickname,nick,sid,host,ip);
				}
			}
		}
		item = linkedlist_NextItem(item);
	}
}
int usr_IsUserInChan(USER userid, CHANNEL vchanid) {
	if (userid == NOUSER) {return -1;}
	if (udat(userid)->pleaseremove == 1) {return -1;}
	if (udat(userid)->authed == 0) {return -1;}
	//we access directly using the linkedlist functions so we do not
	//destroy other functions' cycles
	void* item = udat(userid)->channelinfolist;
	while (item != NULL) {
		struct weircd_user_chaninfo* i = linkedlist_GetData(item);
		if (i != NULL) {
			if ((void*)i->cptr == (void*)vchanid) {
				return 0;
			}
		}
		item = linkedlist_NextItem(item);
	}
	return -1;
}
int usr_ChanUserCount(CHANNEL vchanid) {
	usr_CycleInit();
	int count = 0;
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (usr_IsUserInChan(u,vchanid) == 0) {
			count++;
		}
		u = usr_DoCycle();
	}
	return count;
}
int usr_AmIInvited(USER usr, CHANNEL chan) {
	int r = 0;
	while (r < MAXINVITESPERCHAN) {
		if (cdat(chan)->invite[r] != NULL) {
			if (cdat(chan)->invite[r]->time + INVITELEN >= ts()) {
				if (cdat(chan)->invite[r]->target == usr) {
					return 0;
				}
			}
		}
		r++;
	}
	return -1;
}
void usr_RemoveInvite(USER usr, CHANNEL chan) {
	int r = 0;
	while (r < MAXINVITESPERCHAN) {
		if (cdat(chan)->invite[r] != NULL) {
			if (cdat(chan)->invite[r]->target == usr) {
				chan_FreeInvite(chan,r);
				cdat(chan)->invite[r] = NULL;
			}
		}
		r++;
	}
}
/*void netsplitchan(int id) {
	char send[500];
	//move channel to me
	chans[id]->hopcount = 0;
	chans[id]->servname = NULL;
	snprintf(send,499,"resyncchan :%s",chans[id]->name);send[499] = 0;
	links_Broadcast(send);
}*/
int usr_CheckAndRemoveEmptyChans() {
	chan_CycleInit();
	CHANNEL c = chan_DoCycle();
	while (c != NOCHAN) {
		if (usr_ChanUserCount(c) <= 0) {
			//snprintf(send,499,"delchannel %s",chans[r]->name);send[499] = 0;
			//links_Broadcast(send);
			chan_Delete(c);
			return 0;//call me again!
		}
		c = chan_DoCycle();
	}
	return -1;
}
int usr_IsOppedOnJoin(USER uid, const char* channame) {
	//we don't need uid. we needed for built-in services, but they are a module
	//now and nothing we need to care about. so the parameter is kinda obsolete
	CHANNEL c = chan_FindByName(channame);
	if (c != NOCHAN) {
		return -1;
	}
	if (masklist_CheckItem(noopchanlist,channame) >= 0) {
		return -1;
	}
	return 0;
}
void usr_TellAffected(const char* stuff, USER senduser) {
// IMPORTANT: only for LOCAL users, you are expected to send such important events also to all links manually
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		udat(u)->marked = 0;
		u = usr_DoCycle();
	}
	usr_CycleInit();
	u = usr_DoCycle();
	while (u != NOUSER) {
		if (u != senduser && udat(u)->pleaseremove == 0 && (udat(u)->mode_servicesbot == 1 && udat(senduser)->authed == 1)) {
			if (udat(u)->marked == 0) {
				usnd(u,stuff);
				udat(u)->marked = 1;
			}
		}
		u = usr_DoCycle();
	}
	usr_chans_CycleInit(senduser);
	void* cinfo = usr_chans_DoCycle();
	while (cinfo != NULL) {
		usr_CycleInit();
		u = usr_DoCycle();
		while (u != NOUSER) {
			if (u != senduser && udat(u)->pleaseremove == 0 && udat(u)->mode_servicesbot == 0) {
				if (usr_IsUserInChan(u,usr_chans_GetInfo(cinfo)->cptr) == 0 && udat(u)->hopcount <= 0 && u != senduser) {
					if (udat(u)->marked == 0) {
						usnd(u,stuff);
						udat(u)->marked = 1;
					}
				}
			}
			u = usr_DoCycle();
		}
		cinfo = usr_chans_DoCycle();
	}
}
void userQuit(USER uid, int reason) {
	if (uid == NOUSER) {return;}
	char response[501];
	if (udat(uid)->pleaseremove == 0) {
		snprintf(response,499,":%s!%s@%s QUIT :",udat(uid)->nickname,udat(uid)->suserid,udat(uid)->host);
		if (reason == USERQUIT_QUIT) {
			stradd(response,"Quit: ",499);
			if (udat(uid)->quitreason != NULL) {
				stradd(response,udat(uid)->quitreason,499);
				free(udat(uid)->quitreason);
				udat(uid)->quitreason = NULL;
			}
		}
		if (reason == USERQUIT_CUSTOMREMOTEQUIT) {
			if (udat(uid)->quitreason != NULL) {
				strcopy(response,udat(uid)->quitreason,499);
			}
		}
		if (reason == USERQUIT_LINKCLOSE) {
			stradd(response,"Port is closing down",499);
		}
		if (reason == USERQUIT_TIMEOUT) {
			stradd(response,"Ping timeout",499);
		}
		if (reason == USERQUIT_EOF) {
			stradd(response,"Client quit",499);
		}
		if (reason == USERQUIT_EXCESSFLOOD) {
			stradd(response,"Excess flood",499);
		}
		if (reason == USERQUIT_SENDQEXCEEDED) {
			stradd(response,"SendQ exceeded",499);
		}
		if (reason == USERQUIT_NETSPLIT) {
			if (udat(uid)->quitreason != NULL) {
				stradd(response,udat(uid)->quitreason,499);
			}else{
				stradd(response,"Network split",499);
			}
		}
		if (reason == USERQUIT_COLLISION) {
			stradd(response,"Nick collision",499);
			usnd(uid,"ERROR :Nick collision");
		}
		if (reason == USERQUIT_ERROR) {
			if (udat(uid)->quitreason != NULL) {
				stradd(response,udat(uid)->quitreason,499);
				free(udat(uid)->quitreason);
				udat(uid)->quitreason = NULL;
			}
		}
		usr_TellAffected(response,uid); //will only get sent to local users. so no problem also for collision quits
		
		if (reason != USERQUIT_NETSPLIT // do not broadcast netsplit quits (would result in masses of quit messages), we will instead distribute the real split itself so each server can itself generate the respective quit messages
		&& reason != USERQUIT_COLLISION) { //collision users aren't known to everyone, so we'll also leave this to the linking logic
			if (udat(uid)->hopcount <= 0) {
				//links_Broadcast(response); //broadcast to all links
			}else{
				//links_Forward(response,udat(uid)->servlinkid); //simply forward for non-local users
			}
		}
	}
	if ((udat(uid)->socket < 0) || (reason != USERQUIT_ERROR && reason != USERQUIT_TIMEOUT && reason != USERQUIT_EXCESSFLOOD && reason != USERQUIT_REGISTERTIMEOUT && (reason != USERQUIT_COLLISION || udat(uid)->hopcount > 0))) {
		udat(uid)->pleaseremove = 1;
		udat(uid)->sendbeforeremove = 0;
		usr_CheckAndRemoveEmptyChans();
	}else{
		if (udat(uid)->pleaseremove == 1) {return;}
		udat(uid)->pleaseremove = 1;
		udat(uid)->pleaseremove_time = ts();
		udat(uid)->sendbeforeremove = 1;
		usr_CheckAndRemoveEmptyChans();
	}
}
double cseed (int seed) {
	return seed * (1.0/(RAND_MAX + 1.0));
}
int vrandom(int start, int end) {
	return start + cseed(rand()) * (end - start);
}
/*void forcerename(int uid) {
	char nnick[50];
	unsigned long long k = ts()-vrandom(-5,50);srand(ts());
	while (1) {
		sprintf(&nnick[0],"Guest%ld",k);
		if (ubn(nnick) < 0) {break;}
		k += vrandom(2,7);
	}
	char msg[500];
	snprintf(&msg[0],500,":%s!%s@%s NICK %s",udat(uid)->nickname,udat(uid)->suserid,udat(uid)->host,nnick);
	msg[499] = 0;
	tellaffectedusers(msg,uid);
	usnd(uid,msg);
	strcpy(udat(uid)->nickname,nnick);
	udat(uid)->lastnickchange = ts();
	udat(uid)->needstimeout = 0;
	int gip = getIpID(udat(uid)->ip);
	if (gip >= 0) {ips[gip]->nicklock = ts()+20;}
}*/
void usr_PingAndRemove() {
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (udat(u)->pleaseremove == 1) {
			if (udat(u)->sendbeforeremove == 0 || (udat(u)->pleaseremove_time + 3 < ts() && udat(u)->fakeuser != 0) || udat(u)->sendbuffer[0] == NULL) {
				if (udat(u)->wastedresolverequest || udat(u)->hostresolverequest) {
					if (hostresolv_GetRequestStatus(udat(u)->wastedresolverequest) == RESOLVSTATUS_PENDING
					|| hostresolv_GetRequestStatus(udat(u)->hostresolverequest) == RESOLVSTATUS_PENDING) {
						u = usr_DoCycle();
						continue; //we sadly need to wait for the requests to complete
					}
					if (udat(u)->wastedresolverequest) {hostresolv_FreeRequest(udat(u)->wastedresolverequest);}
					if (udat(u)->hostresolverequest) {hostresolv_FreeRequest(udat(u)->hostresolverequest);}
				}
				if (udat(u)->hopcount <= 0 && udat(u)->socket >= 0) {
					so_CloseSocket(udat(u)->socket);
					udat(u)->socket = -1;
				}
				if (udat(u)->quitreason != NULL) {
					free(udat(u)->quitreason);
					udat(u)->quitreason = NULL;
				}
				if (udat(u)->fakeuser == 1) {
					FakeUsers_DropUserReference(u);
				}
				usr_DelUser(u);
				return;//no secure way to get the next item, so no sense in continuing
			}
		}else{
			char snd[500];
			if (udat(u)->authed == 1) {
				if (udat(u)->hopcount <= 0 && udat(u)->fakeuser == 0) {
					if (udat(u)->ping_waitingforpong <= 0) {
						if (udat(u)->ping_lastping + COMPILEOPTION_PINGFREQUENCY < ts() && udat(u)->hopcount <= 0) {
							udat(u)->ping_pingval = ts();
							sprintf(snd,"PING :%ld",udat(u)->ping_pingval);
							usnd(u,snd);
							udat(u)->ping_waitingforpong = ts();
						}
					}else{
						if (udat(u)->ping_waitingforpong + 40 < ts()) {
							usnd(u,"ERROR :Closing link: ping timeout");
							userQuit(u,USERQUIT_TIMEOUT);
						}
					}
				}
			}else{
				if (udat(u)->CONNECTtime + 15 < ts()) {
					usnd(u,"ERROR :Closing link: Registration timeout");
					userQuit(u,USERQUIT_REGISTERTIMEOUT);
				}
			}
			
			//got a wasted resolve request left
			if (udat(u)->wastedresolverequest) {
				if (hostresolv_GetRequestStatus(udat(u)->wastedresolverequest) != RESOLVSTATUS_PENDING) {
					hostresolv_FreeRequest(udat(u)->wastedresolverequest);
					udat(u)->wastedresolverequest = NULL;
				}
			}
			if (udat(u)->pleaseremove == 0) {
				//check the resolve requests for  this user
				if (udat(u)->hostresolverequest) {
					if (hostresolv_GetRequestStatus(udat(u)->hostresolverequest) == RESOLVSTATUS_FAILURE) {
						if (udat(u)->resolvedhost) { //if forward lookup failed
							free(udat(u)->resolvedhost);
							udat(u)->resolvedhost = NULL;
						}
						usnd(u,"NOTICE AUTH :*** Resolving host failed, falling back to %s",udat(u)->ip);
						udat(u)->resolvestatus = 2;
						hostresolv_FreeRequest(udat(u)->hostresolverequest);
						udat(u)->hostresolverequest = NULL;
						if (udat(u)->authed != 1 && udat(u)->authpongvalue <= 0 && strlen(udat(u)->nickname) > 0 && udat(u)->usercmdissued == 1) {
							//we have the host. finish login
							irc_SendAuthPong(u);
						}
					}else{
						if (hostresolv_GetRequestStatus(udat(u)->hostresolverequest) == RESOLVSTATUS_SUCCESS) {
							if (udat(u)->resolvestatus == 0) {
								//set user host to lookup result
								if (udat(u)->resolvedhost) {free(udat(u)->resolvedhost);}
								const char* p = hostresolv_GetRequestResult(udat(u)->hostresolverequest);
								udat(u)->resolvedhost = malloc(strlen(p)+1);
								if (udat(u)->resolvedhost) {
									strcpy(udat(u)->resolvedhost,p);
								}else{
									usnd(u,"ERROR :Memory allocation problem (host lookup)");
									userQuit(u,USERQUIT_ERROR);
									u = usr_DoCycle();
									continue;
								}
								//update vhost
								strcopy(udat(u)->host,host_GetDefault(u),COMPILEOPTION_MAXHOSTLEN-1);
								//set off forward resolve to confirm host
								hostresolv_FreeRequest(udat(u)->hostresolverequest);
								udat(u)->resolvestatus = 1;
								int wantv6 = 0;
								if (udat(u)->iptype == IPTYPE_IPV6) {wantv6 = 1;}
								udat(u)->hostresolverequest = hostresolv_LookupRequest(udat(u)->resolvedhost, wantv6);
								//set the host in case it's ok (otherwise we'll kill the user later)
								strncpy(udat(u)->host,host_GetDefault(u),COMPILEOPTION_MAXHOSTLEN-1);
								udat(u)->host[COMPILEOPTION_MAXHOSTLEN-1] = 0;
							}else{
								if (udat(u)->resolvestatus == 1) {
									//check whether host is ok
									if (strcasecmp(udat(u)->ip, hostresolv_GetRequestResult(udat(u)->hostresolverequest)) != 0) {
										char tmpresult[200];
										strcopy(tmpresult,hostresolv_GetRequestResult(udat(u)->hostresolverequest),199);
										if (strstw(tmpresult,"[") == 0) {cut(tmpresult,1);}
										if (strendswith(tmpresult,"]") == 0) {tmpresult[strlen(tmpresult)-1] = 0;}
										if (strcasecmp(udat(u)->ip, tmpresult) != 0) {
											usnd(u,"NOTICE AUTH :*** Spoofed host! Using %s instead",udat(u)->ip);
											if (udat(u)->resolvedhost) {
												free(udat(u)->resolvedhost);
												udat(u)->resolvedhost = NULL;
											}
										}
									}
									//complete login if needed
									hostresolv_FreeRequest(udat(u)->hostresolverequest);
									udat(u)->resolvestatus = 2;
									udat(u)->hostresolverequest = NULL;
									if (udat(u)->authed != 1 && udat(u)->authpongvalue <= 0 && strlen(udat(u)->nickname) > 0 && udat(u)->usercmdissued == 1) {
										//we have the host. finish login
										irc_SendAuthPong(u);
									}
								}
							}
						}
					}
				}
			}else{
				//get rid of still running host request
				if (udat(u)->hostresolverequest) {
					if (hostresolv_GetRequestStatus(udat(u)->hostresolverequest) != RESOLVSTATUS_PENDING) {
						hostresolv_FreeRequest(udat(u)->hostresolverequest);
						udat(u)->hostresolverequest = NULL;
					}
				}
			}
		}
		u = usr_DoCycle();
	}
}
void usr_NoticeAllUsers(USER source, const char* stuff) {
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		char msg[513];
		snprintf(msg,512,":%s!%s@%s NOTICE %s :%s",udat(source)->nickname,udat(source)->suserid,udat(source)->host,udat(u)->nickname,stuff);
		msg[512] = 0;
		vsnd(msg,u);
		u = usr_DoCycle();
	}
}
void usr_TellAllUsersInChan(const char* stuff, CHANNEL chanid) {
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (usr_IsUserInChan(u,chanid) == 0) {
			vsnd(stuff,u);
		}
		u = usr_DoCycle();
	}
}
void usr_TellAllUsersInChan_Local(const char* stuff, CHANNEL chanid) {
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (usr_IsUserInChan(u,chanid) == 0 && udat(u)->hopcount <= 0) {
			vsnd(stuff,u);
		}
		u = usr_DoCycle();
	}
}
void usr_TellAllUsersInChanExcept(const char* stuff, CHANNEL chanid, USER excludeuser) {
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (usr_IsUserInChan(u,chanid) == 0 && u != excludeuser) {
			vsnd(stuff,u);
		}
		u = usr_DoCycle();
	}
}
char usr_GetPrefix(USER userid, CHANNEL vchanid) {
	if (userid == NOUSER) {return 0;}
	if (udat(userid)->pleaseremove == 1) {return 0;}
	if (udat(userid)->authed == 0) {return 0;}
	if (usr_IsUserInChan(userid,vchanid) != 0) {return 0;}
	usr_chans_CycleInit(userid);
	void* cinfo = usr_chans_DoCycle();
	while (cinfo != NULL) {
		if (usr_chans_GetInfo(cinfo)->cptr == vchanid) {
			struct weircd_user_chaninfo* ch = usr_chans_GetInfo(cinfo);
			if (ch->iamop == 1) {
				return '@';
			}else{
				if (ch->iamhalfop == 1) {
					return '%';
				}else{
					if (ch->iamvoice == 1) {
						return '+';
					}else{
						return 0;
					}
				}
			}
		}
		cinfo = usr_chans_DoCycle();
	}
	return 0;
}
void IOnotice(const char* str) {
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (udat(u)->pleaseremove == 0 && udat(u)->isircop == 1) {
			usnd(u,":%s NOTICE %s :%s",svname(),udat(u)->nickname,str);
		}
		u = usr_DoCycle();
	}
}
